# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpModifyPolicySetNvtSelectionTestMixin:
    def test_modify_policy_set_nvt_selection(self):
        self.gmp.modify_policy_set_nvt_selection(
            policy_id="c1", family="foo", nvt_oids=["o1"]
        )

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<nvt_selection>"
            b"<family>foo</family>"
            b'<nvt oid="o1"/>'
            b"</nvt_selection>"
            b"</modify_config>"
        )

        self.gmp.modify_policy_set_nvt_selection(
            policy_id="c1", family="foo", nvt_oids=["o1", "o2"]
        )

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<nvt_selection>"
            b"<family>foo</family>"
            b'<nvt oid="o1"/>'
            b'<nvt oid="o2"/>'
            b"</nvt_selection>"
            b"</modify_config>"
        )

        self.gmp.modify_policy_set_nvt_selection("c1", "foo", ["o1"])

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<nvt_selection>"
            b"<family>foo</family>"
            b'<nvt oid="o1"/>'
            b"</nvt_selection>"
            b"</modify_config>"
        )

        self.gmp.modify_policy_set_nvt_selection("c1", "foo", ("o1", "o2"))

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<nvt_selection>"
            b"<family>foo</family>"
            b'<nvt oid="o1"/>'
            b'<nvt oid="o2"/>'
            b"</nvt_selection>"
            b"</modify_config>"
        )

        self.gmp.modify_policy_set_nvt_selection(
            policy_id="c1", family="foo", nvt_oids=[]
        )

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<nvt_selection>"
            b"<family>foo</family>"
            b"</nvt_selection>"
            b"</modify_config>"
        )

    def test_modify_policy_set_nvt_selection_missing_config_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_nvt_selection(
                policy_id=None, family="foo", nvt_oids=["o1"]
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_nvt_selection(
                policy_id="", family="foo", nvt_oids=["o1"]
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_nvt_selection("", "foo", ["o1"])

    def test_modify_policy_set_nvt_selection_missing_family(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_nvt_selection(
                policy_id="c1", family=None, nvt_oids=["o1"]
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_nvt_selection(
                policy_id="c1", family="", nvt_oids=["o1"]
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_nvt_selection("c1", "", ["o1"])

    def test_modify_policy_set_nvt_selection_invalid_nvt_oids(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_nvt_selection(
                policy_id="c1", family="foo", nvt_oids=None
            )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpCreateGroupTestMixin:
    def test_create_group(self):
        self.gmp.create_group(name="foo")

        self.connection.send.has_been_called_with(
            b"<create_group><name>foo</name></create_group>"
        )

    def test_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_group(None)

        with self.assertRaises(RequiredArgument):
            self.gmp.create_group("")

    def test_create_group_with_comment(self):
        self.gmp.create_group(name="foo", comment="bar")

        self.connection.send.has_been_called_with(
            b"<create_group>"
            b"<name>foo</name>"
            b"<comment>bar</comment>"
            b"</create_group>"
        )

    def test_create_special_group(self):
        self.gmp.create_group(name="foo", special=True)

        self.connection.send.has_been_called_with(
            b"<create_group>"
            b"<name>foo</name>"
            b"<specials>"
            b"<full/>"
            b"</specials>"
            b"</create_group>"
        )

    def test_create_group_with_users(self):
        self.gmp.create_group(name="foo", users=[])

        self.connection.send.has_been_called_with(
            b"<create_group><name>foo</name></create_group>"
        )

        self.gmp.create_group(name="foo", users=["u1", "u2"])

        self.connection.send.has_been_called_with(
            b"<create_group><name>foo</name><users>u1,u2</users></create_group>"
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import Roles


class RolesTestCase(unittest.TestCase):
    def test_clone_role(self):
        request = Roles.clone_role("role_id")
        self.assertEqual(
            bytes(request),
            b"<create_role><copy>role_id</copy></create_role>",
        )

    def test_clone_role_missing_role_id(self):
        with self.assertRaises(RequiredArgument):
            Roles.clone_role(None)

        with self.assertRaises(RequiredArgument):
            Roles.clone_role("")

    def test_create_role(self):
        request = Roles.create_role("name")
        self.assertEqual(
            bytes(request),
            b"<create_role><name>name</name></create_role>",
        )

    def test_create_role_with_comment(self):
        request = Roles.create_role("name", comment="comment")
        self.assertEqual(
            bytes(request),
            b"<create_role>"
            b"<name>name</name>"
            b"<comment>comment</comment>"
            b"</create_role>",
        )

    def test_create_role_with_users(self):
        request = Roles.create_role("name", users=["user1", "user2"])
        self.assertEqual(
            bytes(request),
            b"<create_role>"
            b"<name>name</name>"
            b"<users>user1,user2</users>"
            b"</create_role>",
        )

    def test_create_role_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Roles.create_role(None)

        with self.assertRaises(RequiredArgument):
            Roles.create_role("")

    def test_get_roles(self):
        request = Roles.get_roles()
        self.assertEqual(
            bytes(request),
            b"<get_roles/>",
        )

    def test_get_roles_with_filter_string(self):
        request = Roles.get_roles(filter_string="filter_string")
        self.assertEqual(
            bytes(request),
            b'<get_roles filter="filter_string"/>',
        )

    def test_get_roles_with_filter_id(self):
        request = Roles.get_roles(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_roles filt_id="filter_id"/>',
        )

    def test_get_roles_with_trash(self):
        request = Roles.get_roles(trash=True)
        self.assertEqual(
            bytes(request),
            b'<get_roles trash="1"/>',
        )

        request = Roles.get_roles(trash=False)
        self.assertEqual(
            bytes(request),
            b'<get_roles trash="0"/>',
        )

    def test_get_role(self):
        request = Roles.get_role("role_id")
        self.assertEqual(
            bytes(request),
            b'<get_roles role_id="role_id"/>',
        )

    def test_get_role_missing_role_id(self):
        with self.assertRaises(RequiredArgument):
            Roles.get_role(None)

        with self.assertRaises(RequiredArgument):
            Roles.get_role("")

    def test_modify_role(self):
        request = Roles.modify_role("role_id")
        self.assertEqual(
            bytes(request),
            b'<modify_role role_id="role_id"/>',
        )

    def test_modify_role_with_comment(self):
        request = Roles.modify_role("role_id", comment="comment")
        self.assertEqual(
            bytes(request),
            b'<modify_role role_id="role_id">'
            b"<comment>comment</comment>"
            b"</modify_role>",
        )

    def test_modify_role_with_name(self):
        request = Roles.modify_role("role_id", name="name")
        self.assertEqual(
            bytes(request),
            b'<modify_role role_id="role_id"><name>name</name></modify_role>',
        )

    def test_modify_role_with_users(self):
        request = Roles.modify_role("role_id", users=["user1", "user2"])
        self.assertEqual(
            bytes(request),
            b'<modify_role role_id="role_id">'
            b"<users>user1,user2</users>"
            b"</modify_role>",
        )

    def test_modify_role_missing_role_id(self):
        with self.assertRaises(RequiredArgument):
            Roles.modify_role(None)

        with self.assertRaises(RequiredArgument):
            Roles.modify_role("")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import Groups


class GroupsTestCase(unittest.TestCase):
    def test_clone_group(self):
        request = Groups.clone_group("group_id")
        self.assertEqual(
            bytes(request),
            b"<create_group><copy>group_id</copy></create_group>",
        )

    def test_clone_group_missing_group_id(self):
        with self.assertRaises(RequiredArgument):
            Groups.clone_group(None)

        with self.assertRaises(RequiredArgument):
            Groups.clone_group("")

    def test_create_group(self):
        request = Groups.create_group("group_name")
        self.assertEqual(
            bytes(request),
            b"<create_group><name>group_name</name></create_group>",
        )

    def test_create_group_with_comment(self):
        request = Groups.create_group("group_name", comment="comment")
        self.assertEqual(
            bytes(request),
            b"<create_group>"
            b"<name>group_name</name>"
            b"<comment>comment</comment>"
            b"</create_group>",
        )

    def test_create_group_with_special(self):
        request = Groups.create_group("group_name", special=True)
        self.assertEqual(
            bytes(request),
            b"<create_group>"
            b"<name>group_name</name>"
            b"<specials><full/></specials>"
            b"</create_group>",
        )

        request = Groups.create_group("group_name", special=False)
        self.assertEqual(
            bytes(request),
            b"<create_group><name>group_name</name></create_group>",
        )

    def test_create_group_with_users(self):
        request = Groups.create_group("group_name", users=["user1", "user2"])
        self.assertEqual(
            bytes(request),
            b"<create_group>"
            b"<name>group_name</name>"
            b"<users>user1,user2</users>"
            b"</create_group>",
        )

    def test_create_group_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Groups.create_group(None)

        with self.assertRaises(RequiredArgument):
            Groups.create_group("")

    def test_delete_group(self):
        request = Groups.delete_group("group_id")
        self.assertEqual(
            bytes(request),
            b'<delete_group group_id="group_id" ultimate="0"/>',
        )

    def test_delete_group_with_ultimate(self):
        request = Groups.delete_group("group_id", ultimate=True)
        self.assertEqual(
            bytes(request),
            b'<delete_group group_id="group_id" ultimate="1"/>',
        )

        request = Groups.delete_group("group_id", ultimate=False)
        self.assertEqual(
            bytes(request),
            b'<delete_group group_id="group_id" ultimate="0"/>',
        )

    def test_delete_group_missing_group_id(self):
        with self.assertRaises(RequiredArgument):
            Groups.delete_group(None)

        with self.assertRaises(RequiredArgument):
            Groups.delete_group("")

    def test_get_groups(self):
        request = Groups.get_groups()
        self.assertEqual(
            bytes(request),
            b"<get_groups/>",
        )

    def test_get_groups_with_filter_string(self):
        request = Groups.get_groups(filter_string="filter_string")
        self.assertEqual(
            bytes(request),
            b'<get_groups filter="filter_string"/>',
        )

    def test_get_groups_with_filter_id(self):
        request = Groups.get_groups(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_groups filt_id="filter_id"/>',
        )

    def test_get_groups_with_trash(self):
        request = Groups.get_groups(trash=True)
        self.assertEqual(
            bytes(request),
            b'<get_groups trash="1"/>',
        )

        request = Groups.get_groups(trash=False)
        self.assertEqual(
            bytes(request),
            b'<get_groups trash="0"/>',
        )

    def test_get_group(self):
        request = Groups.get_group("group_id")
        self.assertEqual(
            bytes(request),
            b'<get_groups group_id="group_id"/>',
        )

    def test_group_missing_group_id(self):
        with self.assertRaises(RequiredArgument):
            Groups.get_group(None)

        with self.assertRaises(RequiredArgument):
            Groups.get_group("")

    def test_modify_group(self):
        request = Groups.modify_group("group_id")
        self.assertEqual(
            bytes(request),
            b'<modify_group group_id="group_id"/>',
        )

    def test_modify_group_with_comment(self):
        request = Groups.modify_group("group_id", comment="comment")
        self.assertEqual(
            bytes(request),
            b'<modify_group group_id="group_id">'
            b"<comment>comment</comment>"
            b"</modify_group>",
        )

    def test_modify_group_with_name(self):
        request = Groups.modify_group("group_id", name="name")
        self.assertEqual(
            bytes(request),
            b'<modify_group group_id="group_id">'
            b"<name>name</name>"
            b"</modify_group>",
        )

    def test_modify_group_with_users(self):
        request = Groups.modify_group("group_id", users=["user1", "user2"])
        self.assertEqual(
            bytes(request),
            b'<modify_group group_id="group_id">'
            b"<users>user1,user2</users>"
            b"</modify_group>",
        )

    def test_modify_group_missing_group_id(self):
        with self.assertRaises(RequiredArgument):
            Groups.modify_group(None)

        with self.assertRaises(RequiredArgument):
            Groups.modify_group("")



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""
API wrapper for interacting with the Notus component of the openvasd HTTP API.
"""

import urllib

import httpx

from ._api import OpenvasdAPI


class NotusAPI(OpenvasdAPI):
    """
    Provides access to the Notus-related endpoints of the openvasd HTTP API.

    This includes retrieving supported operating systems and triggering
    package-based vulnerability scans for a specific OS.
    """

    def get_os_list(self) -> httpx.Response:
        """
        Retrieve the list of supported operating systems from the Notus service.

        Returns:
            The full `httpx.Response` on success.

        See: GET /notus in the openvasd API documentation.
        """
        try:
            response = self._client.get("/notus")
            response.raise_for_status()
            return response
        except httpx.HTTPStatusError as e:
            if self._suppress_exceptions:
                return e.response
            raise

    def run_scan(
        self,
        os: str,
        package_list: list[str],
    ) -> httpx.Response:
        """
        Trigger a Notus scan for a given OS and list of packages.

        Args:
            os: Operating system name (e.g., "debian", "alpine").
            package_list: List of package names to evaluate for vulnerabilities.

        Returns:
            The full `httpx.Response` on success.

        See: POST /notus/{os} in the openvasd API documentation.
        """
        quoted_os = urllib.parse.quote(os)
        try:
            response = self._client.post(
                f"/notus/{quoted_os}", json=package_list
            )
            response.raise_for_status()
            return response
        except httpx.HTTPStatusError as e:
            if self._suppress_exceptions:
                return e.response
            raise



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from lxml import etree

from gvm.errors import GvmError, GvmResponseError, GvmServerError
from gvm.transforms import CheckCommandTransform


class CheckCommandTransformTestCase(unittest.TestCase):
    def test_no_status_transform(self):
        transform = CheckCommandTransform()

        with self.assertRaises(GvmError):
            transform(b"<foo/>")

    def test_no_success_300status_transform(self):
        transform = CheckCommandTransform()

        root = etree.Element("foo_response")
        root.set("status", "300")
        root.set("status_text", "Foo error")

        response = etree.tostring(root)

        with self.assertRaises(GvmError):
            transform(response)

    def test_no_success_400status_transform(self):
        transform = CheckCommandTransform()

        root = etree.Element("foo_response")
        root.set("status", "400")
        root.set("status_text", "Foo error")

        response = etree.tostring(root)

        with self.assertRaises(GvmResponseError):
            transform(response)

    def test_no_success_500status_transform(self):
        transform = CheckCommandTransform()

        root = etree.Element("foo_response")
        root.set("status", "500")
        root.set("status_text", "Foo error")

        response = etree.tostring(root)

        with self.assertRaises(GvmServerError):
            transform(response)

    def test_success_response(self):
        transform = CheckCommandTransform()

        root = etree.Element("foo_response")
        root.set("status", "200")

        response = etree.tostring(root)

        self.assertEqual(transform(response), b'<foo_response status="200"/>')



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.protocols.core import Response, StatusError
from gvm.xml import Element, XmlError


class RequestMock:
    def __init__(self, data: bytes) -> None:
        self._data = data

    def __bytes__(self) -> bytes:
        return self._data


class ResponseTestCase(unittest.TestCase):
    def test_data(self) -> None:
        request = RequestMock(b"<request/>")
        response = Response(data=b"<response/>", request=request)

        self.assertEqual(response.data, b"<response/>")

    def test_bytes(self) -> None:
        request = RequestMock(b"<request/>")
        response = Response(data=b"<response/>", request=request)

        self.assertEqual(bytes(response), b"<response/>")

    def test_request(self) -> None:
        request = RequestMock(b"<request/>")
        response = Response(data=b"<response/>", request=request)

        self.assertEqual(response.request, request)

    def test_status_code(self):
        request = RequestMock(b"<request/>")
        response = Response(data=b"<response/>", request=request)
        self.assertIsNone(response.status_code)

        response = Response(data=b'<response status="abc"/>', request=request)
        self.assertIsNone(response.status_code)

        response = Response(data=b'<response status="123"/>', request=request)
        self.assertEqual(response.status_code, 123)

    def test_is_success(self):
        request = RequestMock(b"<request/>")
        response = Response(data=b"<response/>", request=request)
        self.assertFalse(response.is_success)

        response = Response(data=b'<response status="abc"/>', request=request)
        self.assertFalse(response.is_success)

        response = Response(data=b'<response status="123"/>', request=request)
        self.assertFalse(response.is_success)

        response = Response(data=b'<response status="234"/>', request=request)
        self.assertTrue(response.is_success)

    def test_raise_for_status(self):
        request = RequestMock(b"<request/>")
        response = Response(data=b"<response/>", request=request)

        with self.assertRaisesRegex(StatusError, "^Invalid status code None$"):
            response.raise_for_status()

        response = Response(data=b'<response status="abc"/>', request=request)
        with self.assertRaisesRegex(StatusError, "^Invalid status code None$"):
            response.raise_for_status()

        response = Response(data=b'<response status="123"/>', request=request)
        with self.assertRaisesRegex(StatusError, "^Invalid status code 123$"):
            response.raise_for_status()

        response = Response(data=b'<response status="234"/>', request=request)
        self.assertIs(response.raise_for_status(), response)

    def test_xml(self):
        request = RequestMock(b"<request/>")
        response = Response(
            data=b'<response><some data="lorem"/></response>', request=request
        )

        xml = response.xml()
        self.assertIsInstance(xml, Element)

        self.assertEqual(xml.tag, "response")
        sub_element = xml[0]
        self.assertEqual(sub_element.tag, "some")
        self.assertEqual(sub_element.attrib["data"], "lorem")

    def test_invalid_xml(self):
        request = RequestMock(b"<request/>")
        response = Response(data=b"<response</response>", request=request)

        with self.assertRaisesRegex(
            XmlError, "^Invalid XML b'<response</response>'. Error was .*$"
        ):
            response.xml()



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class Schedules:
    @classmethod
    def clone_schedule(cls, schedule_id: EntityID) -> Request:
        """Clone an existing schedule

        Args:
            schedule_id: UUID of an existing schedule to clone from
        """
        if not schedule_id:
            raise RequiredArgument(
                function=cls.clone_schedule.__name__, argument="schedule_id"
            )

        cmd = XmlCommand("create_schedule")
        cmd.add_element("copy", str(schedule_id))
        return cmd

    @classmethod
    def create_schedule(
        cls,
        name: str,
        icalendar: str,
        timezone: str,
        *,
        comment: str | None = None,
    ) -> Request:
        """Create a new schedule based in `iCalendar`_ data.

        Example:
            Requires https://pypi.org/project/icalendar/

            .. code-block:: python

                import pytz

                from datetime import datetime

                from icalendar import Calendar, Event

                cal = Calendar()

                cal.add("prodid", "-//Foo Bar//")
                cal.add("version", "2.0")

                event = Event()
                event.add("dtstamp", datetime.now(tz=pytz.UTC))
                event.add("dtstart", datetime(2020, 1, 1, tzinfo=pytz.utc))

                cal.add_component(event)

                Schedules.create_schedule(
                    name="My Schedule", icalendar=cal.to_ical(), timezone="UTC"
                )

        Args:
            name: Name of the new schedule
            icalendar: `iCalendar`_ (RFC 5545) based data.
            timezone: Timezone to use for the icalendar events e.g
                Europe/Berlin. If the datetime values in the icalendar data are
                missing timezone information this timezone gets applied.
                Otherwise the datetime values from the icalendar data are
                displayed in this timezone
            comment: Comment on schedule.

        .. _iCalendar:
            https://tools.ietf.org/html/rfc5545
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_schedule.__name__, argument="name"
            )

        if not icalendar:
            raise RequiredArgument(
                function=cls.create_schedule.__name__, argument="icalendar"
            )

        if not timezone:
            raise RequiredArgument(
                function=cls.create_schedule.__name__, argument="timezone"
            )

        cmd = XmlCommand("create_schedule")

        cmd.add_element("name", name)
        cmd.add_element("icalendar", icalendar)
        cmd.add_element("timezone", timezone)

        if comment:
            cmd.add_element("comment", comment)

        return cmd

    @classmethod
    def delete_schedule(
        cls, schedule_id: EntityID, *, ultimate: bool | None = False
    ) -> Request:
        """Deletes an existing schedule

        Args:
            schedule_id: UUID of the schedule to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not schedule_id:
            raise RequiredArgument(
                function=cls.delete_schedule.__name__, argument="schedule_id"
            )

        cmd = XmlCommand("delete_schedule")
        cmd.set_attribute("schedule_id", str(schedule_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))

        return cmd

    @staticmethod
    def get_schedules(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        tasks: bool | None = None,
    ) -> Request:
        """Request a list of schedules

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get the trashcan schedules instead
            tasks: Whether to include tasks using the schedules
        """
        cmd = XmlCommand("get_schedules")

        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        if tasks is not None:
            cmd.set_attribute("tasks", to_bool(tasks))

        return cmd

    @classmethod
    def get_schedule(
        cls, schedule_id: EntityID, *, tasks: bool | None = None
    ) -> Request:
        """Request a single schedule

        Args:
            schedule_id: UUID of an existing schedule
            tasks: Whether to include tasks using the schedules
        """
        cmd = XmlCommand("get_schedules")

        if not schedule_id:
            raise RequiredArgument(
                function=cls.get_schedule.__name__, argument="schedule_id"
            )

        cmd.set_attribute("schedule_id", str(schedule_id))

        if tasks is not None:
            cmd.set_attribute("tasks", to_bool(tasks))

        return cmd

    @classmethod
    def modify_schedule(
        cls,
        schedule_id: EntityID,
        *,
        name: str | None = None,
        icalendar: str | None = None,
        timezone: str | None = None,
        comment: str | None = None,
    ) -> Request:
        """Modifies an existing schedule

        Args:
            schedule_id: UUID of the schedule to be modified
            name: Name of the schedule
            icalendar: `iCalendar`_ (RFC 5545) based data.
            timezone: Timezone to use for the icalendar events e.g
                Europe/Berlin. If the datetime values in the icalendar data are
                missing timezone information this timezone gets applied.
                Otherwise the datetime values from the icalendar data are
                displayed in this timezone
            comment: Comment on schedule.

        .. _iCalendar:
            https://tools.ietf.org/html/rfc5545
        """
        if not schedule_id:
            raise RequiredArgument(
                function=cls.modify_schedule.__name__, argument="schedule_id"
            )

        cmd = XmlCommand("modify_schedule")
        cmd.set_attribute("schedule_id", str(schedule_id))

        if name:
            cmd.add_element("name", name)

        if icalendar:
            cmd.add_element("icalendar", icalendar)

        if timezone:
            cmd.add_element("timezone", timezone)

        if comment:
            cmd.add_element("comment", comment)

        return cmd



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm._enum import Enum
from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.core import Request
from gvm.xml import XmlCommand

from .._entity_id import EntityID
from ..v224._credentials import (
    Credentials as CredentialsV224,
)


class CredentialStoreCredentialType(Enum):
    """Enum for credential store credential types"""

    CLIENT_CERTIFICATE = "cs_cc"
    SNMP = "cs_snmp"
    USERNAME_PASSWORD = "cs_up"
    USERNAME_SSH_KEY = "cs_usk"
    SMIME_CERTIFICATE = "cs_smime"
    PGP_ENCRYPTION_KEY = "cs_pgp"
    PASSWORD_ONLY = "cs_pw"


class Credentials(CredentialsV224):
    @classmethod
    def create_credential_store_credential(
        cls,
        name: str,
        credential_type: CredentialStoreCredentialType | str,
        *,
        comment: str | None = None,
        credential_store_id: EntityID | None = None,
        vault_id: str | None = None,
        host_identifier: str | None = None,
    ) -> Request:
        """Create a new credential that is fetched from a credential store

        Create a new credential e.g. to be used in the method of an alert.

        Currently the following credential types are supported:

            - Username + Password
            - Username + SSH-Key
            - Client Certificates
            - SNMPv1 or SNMPv2c protocol
            - S/MIME Certificate
            - OpenPGP Key
            - Password only

        Arguments:
            name: Name of the new credential
            credential_type: The credential type.
            comment: Comment for the credential
            credential_store_id: Optional id of the credential store to use
                (gvmd will pick default one if none is provided)
            vault_id: Vault-ID used to access the secret in credential store
            host_identifier: Host-Identifier used to access the secret in credential store

        Examples:
            Creating a Password-Only credential stored in a Credential Store

            .. code-block:: python

                request = Credentials.create_credential(
                    name="Credential-Store Password-Only Credential",
                    credential_type=CredentialType.CREDENTIAL_STOREPASSWORD_ONLY,
                    vault_id="a5f84dd4-da18-447c-a9fb-b77b5df49076",
                    host_identifier="/My/Secret",
                )
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_credential.__name__, argument="name"
            )

        if not credential_type:
            raise RequiredArgument(
                function=cls.create_credential.__name__,
                argument="credential_type",
            )

        if not isinstance(credential_type, CredentialStoreCredentialType):
            credential_type = CredentialStoreCredentialType(credential_type)

        cmd = XmlCommand("create_credential")
        cmd.add_element("name", name)

        cmd.add_element("type", credential_type.value)

        if comment:
            cmd.add_element("comment", comment)

        if (
            credential_type != CredentialStoreCredentialType.CLIENT_CERTIFICATE
            and credential_type != CredentialStoreCredentialType.SNMP
            and credential_type
            != CredentialStoreCredentialType.USERNAME_PASSWORD
            and credential_type
            != CredentialStoreCredentialType.USERNAME_SSH_KEY
            and credential_type
            != CredentialStoreCredentialType.SMIME_CERTIFICATE
            and credential_type
            != CredentialStoreCredentialType.PGP_ENCRYPTION_KEY
            and credential_type != CredentialStoreCredentialType.PASSWORD_ONLY
        ):
            raise InvalidArgument(
                function=cls.create_credential.__name__,
                argument="credential_type",
            )

        if not vault_id:
            raise RequiredArgument(
                function=cls.create_credential.__name__,
                argument="vault_id",
            )
        if not host_identifier:
            raise RequiredArgument(
                function=cls.create_credential.__name__,
                argument="host_identifier",
            )

        if credential_store_id:
            cmd.add_element("credential_store_id", str(credential_store_id))

        cmd.add_element("vault_id", vault_id)
        cmd.add_element("host_identifier", host_identifier)

        return cmd

    @classmethod
    def modify_credential_store_credential(
        cls,
        credential_id: EntityID,
        *,
        name: str | None = None,
        comment: str | None = None,
        credential_store_id: EntityID | None = None,
        vault_id: str | None = None,
        host_identifier: str | None = None,
    ) -> Request:
        """Modifies an existing credential.

        Arguments:
            credential_id: UUID of the credential
            name: Name of the credential
            comment: Comment for the credential
            credential_store_id: Optional id of the credential store to use
                (gvmd will pick default one if none is provided)
            vault_id: Vault-ID used to access the secret in credential store
            host_identifier: Host-Identifier used to access the secret in credential store
        """
        if not credential_id:
            raise RequiredArgument(
                function=cls.modify_credential.__name__,
                argument="credential_id",
            )

        cmd = XmlCommand("modify_credential")
        cmd.set_attribute("credential_id", str(credential_id))

        if name:
            cmd.add_element("name", name)
        if comment:
            cmd.add_element("comment", comment)

        if credential_store_id:
            cmd.add_element("credential_store_id", str(credential_store_id))
        if vault_id:
            cmd.add_element("vault_id", vault_id)
        if host_identifier:
            cmd.add_element("host_identifier", host_identifier)

        return cmd



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetCredentialsTestMixin:
    def test_get_credentials(self):
        self.gmp.get_credentials()

        self.connection.send.has_been_called_with(b"<get_credentials/>")

    def test_get_credentials_with_filter_string(self):
        self.gmp.get_credentials(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_credentials filter="foo=bar"/>'
        )

    def test_get_credentials_with_filter_id(self):
        self.gmp.get_credentials(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_credentials filt_id="f1"/>'
        )

    def test_get_credentials_with_scanners(self):
        self.gmp.get_credentials(scanners=True)

        self.connection.send.has_been_called_with(
            b'<get_credentials scanners="1"/>'
        )

        self.gmp.get_credentials(scanners=False)

        self.connection.send.has_been_called_with(
            b'<get_credentials scanners="0"/>'
        )

    def test_get_credentials_with_trash(self):
        self.gmp.get_credentials(trash=True)

        self.connection.send.has_been_called_with(
            b'<get_credentials trash="1"/>'
        )

        self.gmp.get_credentials(trash=False)

        self.connection.send.has_been_called_with(
            b'<get_credentials trash="0"/>'
        )

    def test_get_credentials_with_targets(self):
        self.gmp.get_credentials(targets=True)

        self.connection.send.has_been_called_with(
            b'<get_credentials targets="1"/>'
        )

        self.gmp.get_credentials(targets=False)

        self.connection.send.has_been_called_with(
            b'<get_credentials targets="0"/>'
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class Nvts:
    @staticmethod
    def get_nvt_families(*, sort_order: str | None = None) -> Request:
        """Request a list of nvt families

        Args:
            sort_order: Sort order
        """
        cmd = XmlCommand("get_nvt_families")

        if sort_order:
            cmd.set_attribute("sort_order", sort_order)

        return cmd

    @staticmethod
    def get_scan_config_nvts(
        *,
        details: bool | None = None,
        preferences: bool | None = None,
        preference_count: bool | None = None,
        timeout: bool | None = None,
        config_id: EntityID | None = None,
        preferences_config_id: EntityID | None = None,
        family: str | None = None,
        sort_order: str | None = None,
        sort_field: str | None = None,
    ) -> Request:
        """Request a list of nvts

        Args:
            details: Whether to include full details
            preferences: Whether to include nvt preferences
            preference_count: Whether to include preference count
            timeout: Whether to include the special timeout preference
            config_id: UUID of scan config to which to limit the NVT listing
            preferences_config_id: UUID of scan config to use for preference
                values
            family: Family to which to limit NVT listing
            sort_order: Sort order
            sort_field: Sort field
        """
        cmd = XmlCommand("get_nvts")

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        if preferences is not None:
            cmd.set_attribute("preferences", to_bool(preferences))

        if preference_count is not None:
            cmd.set_attribute("preference_count", to_bool(preference_count))

        if timeout is not None:
            cmd.set_attribute("timeout", to_bool(timeout))

        if config_id:
            cmd.set_attribute("config_id", str(config_id))

        if preferences_config_id:
            cmd.set_attribute(
                "preferences_config_id", str(preferences_config_id)
            )

        if family:
            cmd.set_attribute("family", family)

        if sort_order:
            cmd.set_attribute("sort_order", sort_order)

        if sort_field:
            cmd.set_attribute("sort_field", sort_field)

        return cmd

    @classmethod
    def get_scan_config_nvt(cls, nvt_oid: str) -> Request:
        """Request a single nvt

        Args:
            nvt_oid: OID of an existing nvt
        """
        cmd = XmlCommand("get_nvts")

        if not nvt_oid:
            raise RequiredArgument(
                function=cls.get_scan_config_nvt.__name__, argument="nvt_oid"
            )

        cmd.set_attribute("nvt_oid", nvt_oid)

        # for single entity always request all details
        cmd.set_attribute("details", "1")
        cmd.set_attribute("preferences", "1")
        cmd.set_attribute("preference_count", "1")

        return cmd

    @classmethod
    def get_nvts(
        cls,
        *,
        filter_string: str | None = None,
        filter_id: str | None = None,
        name: str | None = None,
        details: bool | None = None,
        extended: bool | None = None,
        preferences: bool | None = None,
        preference_count: bool | None = None,
        timeout: bool | None = None,
        config_id: str | None = None,
        preferences_config_id: str | None = None,
        family: str | None = None,
        sort_order: str | None = None,
        sort_field: str | None = None,
    ) -> Request:
        """Request a list of NVTs

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            name: Name or identifier of the requested information
            details: Whether to include information about references to this
                information
            extended: Whether to receive extended NVT information
                (calls get_nvts, instead of get_info)
            preferences: Whether to include NVT preferences (only for extended)
            preference_count: Whether to include preference count (only for extended)
            timeout: Whether to include the special timeout preference (only for extended)
            config_id: UUID of scan config to which to limit the NVT listing (only for extended)
            preferences_config_id: UUID of scan config to use for preference
                values (only for extended)
            family: Family to which to limit NVT listing (only for extended)
            sort_order: Sort order (only for extended)
            sort_field: Sort field (only for extended)
        """
        if extended:
            return cls.get_scan_config_nvts(
                details=details,
                preferences=preferences,
                preference_count=preference_count,
                timeout=timeout,
                config_id=config_id,
                preferences_config_id=preferences_config_id,
                family=family,
                sort_order=sort_order,
                sort_field=sort_field,
            )

        cmd = XmlCommand("get_info")

        cmd.set_attribute("type", "NVT")

        cmd.add_filter(filter_string, filter_id)

        if name:
            cmd.set_attribute("name", name)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        return cmd

    @classmethod
    def get_nvt(cls, nvt_id: str, *, extended: bool | None = None) -> Request:
        """Request a single NVT

        Args:
            nvt_id: ID of an existing NVT
            extended: Whether to receive extended NVT information
                (calls get_nvts, instead of get_info)
        """
        if not nvt_id:
            raise RequiredArgument(
                function=cls.get_nvt.__name__, argument="nvt_id"
            )

        if extended:
            return cls.get_scan_config_nvt(nvt_id)

        cmd = XmlCommand("get_info")
        cmd.set_attribute("info_id", nvt_id)

        cmd.set_attribute("type", "NVT")

        # for single entity always request all details
        cmd.set_attribute("details", "1")
        return cmd

    @staticmethod
    def get_nvt_preferences(
        *,
        nvt_oid: str | None = None,
    ) -> Request:
        """Request a list of preferences

        The preference element includes just the
        name and value, with the NVT and type built into the name.

        Args:
            nvt_oid: OID of nvt
        """
        cmd = XmlCommand("get_preferences")

        if nvt_oid:
            cmd.set_attribute("nvt_oid", nvt_oid)

        return cmd

    @classmethod
    def get_nvt_preference(
        cls,
        name: str,
        *,
        nvt_oid: str | None = None,
    ) -> Request:
        """Request a nvt preference

        Args:
            name: name of a particular preference
            nvt_oid: OID of nvt
            config_id: UUID of scan config of which to show preference values
        """
        cmd = XmlCommand("get_preferences")

        if not name:
            raise RequiredArgument(
                function=cls.get_nvt_preference.__name__, argument="name"
            )

        cmd.set_attribute("preference", name)

        if nvt_oid:
            cmd.set_attribute("nvt_oid", nvt_oid)

        return cmd



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v226 import Reports

REPORT_XML_STRING = (
    '<report id="67a62fb7-b238-4f0e-bc48-59bde8939cdc">'
    '<results max="1" start="1">'
    '<result id="f180b40f-49dd-4856-81ed-8c1195afce80">'
    "<severity>0.0</severity>"
    '<nvt oid="1.3.6.1.4.1.25623.1.0.10330"/>'
    "<host>132.67.253.114</host>"
    "</result></results></report>"
)


class ReportsTestCase(unittest.TestCase):
    def test_delete_report(self):
        request = Reports.delete_report("report_id")
        self.assertEqual(
            bytes(request),
            b'<delete_report report_id="report_id"/>',
        )

    def test_delete_report_missing_report_id(self):
        with self.assertRaises(RequiredArgument):
            Reports.delete_report(None)

        with self.assertRaises(RequiredArgument):
            Reports.delete_report("")

    def test_get_report(self):
        request = Reports.get_report("report_id")
        self.assertEqual(
            bytes(request),
            b'<get_reports report_id="report_id" usage_type="scan" details="1"/>',
        )

    def test_get_report_with_filter_string(self):
        request = Reports.get_report("report_id", filter_string="filter_string")
        self.assertEqual(
            bytes(request),
            b'<get_reports report_id="report_id" usage_type="scan" filter="filter_string" details="1"/>',
        )

    def test_get_report_with_filter_id(self):
        request = Reports.get_report("report_id", filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_reports report_id="report_id" usage_type="scan" filt_id="filter_id" details="1"/>',
        )

    def test_get_report_with_delta_report_id(self):
        request = Reports.get_report(
            "report_id", delta_report_id="delta_report_id"
        )
        self.assertEqual(
            bytes(request),
            b'<get_reports report_id="report_id" usage_type="scan" delta_report_id="delta_report_id" details="1"/>',
        )

    def test_get_report_with_report_format_id(self):
        request = Reports.get_report(
            "report_id", report_format_id="report_format_id"
        )
        self.assertEqual(
            bytes(request),
            b'<get_reports report_id="report_id" usage_type="scan" format_id="report_format_id" details="1"/>',
        )

    def test_get_report_with_report_format_id_and_config_id(self):
        request = Reports.get_report(
            "report_id",
            report_format_id="report_format_id",
            report_config_id="report_config_id",
        )
        self.assertEqual(
            bytes(request),
            b'<get_reports report_id="report_id" usage_type="scan" format_id="report_format_id" config_id="report_config_id" details="1"/>',
        )

    def test_get_report_with_ignore_pagination(self):
        request = Reports.get_report("report_id", ignore_pagination=True)
        self.assertEqual(
            bytes(request),
            b'<get_reports report_id="report_id" usage_type="scan" ignore_pagination="1" details="1"/>',
        )

    def test_get_report_missing_report_id(self):
        with self.assertRaises(RequiredArgument):
            Reports.get_report(None)

        with self.assertRaises(RequiredArgument):
            Reports.get_report("")

    def test_get_reports(self):
        request = Reports.get_reports()
        self.assertEqual(
            bytes(request),
            b'<get_reports usage_type="scan"/>',
        )

    def test_get_reports_with_filter_string(self):
        request = Reports.get_reports(filter_string="filter_string")
        self.assertEqual(
            bytes(request),
            b'<get_reports usage_type="scan" report_filter="filter_string"/>',
        )

    def test_get_reports_with_filter_id(self):
        request = Reports.get_reports(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_reports usage_type="scan" report_filt_id="filter_id"/>',
        )

    def test_get_reports_with_note_details(self):
        request = Reports.get_reports(note_details=True)
        self.assertEqual(
            bytes(request),
            b'<get_reports usage_type="scan" note_details="1"/>',
        )

        request = Reports.get_reports(note_details=False)
        self.assertEqual(
            bytes(request),
            b'<get_reports usage_type="scan" note_details="0"/>',
        )

    def test_get_reports_with_override_details(self):
        request = Reports.get_reports(override_details=True)
        self.assertEqual(
            bytes(request),
            b'<get_reports usage_type="scan" override_details="1"/>',
        )

        request = Reports.get_reports(override_details=False)
        self.assertEqual(
            bytes(request),
            b'<get_reports usage_type="scan" override_details="0"/>',
        )

    def test_get_reports_with_details(self):
        request = Reports.get_reports(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_reports usage_type="scan" details="1"/>',
        )

        request = Reports.get_reports(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_reports usage_type="scan" details="0"/>',
        )

    def test_get_reports_with_ignore_pagination(self):
        request = Reports.get_reports(ignore_pagination=True)
        self.assertEqual(
            bytes(request),
            b'<get_reports usage_type="scan" ignore_pagination="1"/>',
        )

        request = Reports.get_reports(ignore_pagination=False)
        self.assertEqual(
            bytes(request),
            b'<get_reports usage_type="scan" ignore_pagination="0"/>',
        )

    def test_import_report(self):
        request = Reports.import_report(REPORT_XML_STRING, "task_id")
        self.assertEqual(
            bytes(request),
            b"<create_report>"
            b'<task id="task_id"/>'
            b'<report id="67a62fb7-b238-4f0e-bc48-59bde8939cdc">'
            b'<results max="1" start="1">'
            b'<result id="f180b40f-49dd-4856-81ed-8c1195afce80">'
            b"<severity>0.0</severity>"
            b'<nvt oid="1.3.6.1.4.1.25623.1.0.10330"/>'
            b"<host>132.67.253.114</host>"
            b"</result></results></report>"
            b"</create_report>",
        )

    def test_import_report_with_in_assets(self):
        request = Reports.import_report(
            REPORT_XML_STRING, "task_id", in_assets=False
        )
        self.assertEqual(
            bytes(request),
            b"<create_report>"
            b'<task id="task_id"/>'
            b"<in_assets>0</in_assets>"
            b'<report id="67a62fb7-b238-4f0e-bc48-59bde8939cdc">'
            b'<results max="1" start="1">'
            b'<result id="f180b40f-49dd-4856-81ed-8c1195afce80">'
            b"<severity>0.0</severity>"
            b'<nvt oid="1.3.6.1.4.1.25623.1.0.10330"/>'
            b"<host>132.67.253.114</host>"
            b"</result></results></report>"
            b"</create_report>",
        )

        request = Reports.import_report(
            REPORT_XML_STRING, "task_id", in_assets=True
        )
        self.assertEqual(
            bytes(request),
            b"<create_report>"
            b'<task id="task_id"/>'
            b"<in_assets>1</in_assets>"
            b'<report id="67a62fb7-b238-4f0e-bc48-59bde8939cdc">'
            b'<results max="1" start="1">'
            b'<result id="f180b40f-49dd-4856-81ed-8c1195afce80">'
            b"<severity>0.0</severity>"
            b'<nvt oid="1.3.6.1.4.1.25623.1.0.10330"/>'
            b"<host>132.67.253.114</host>"
            b"</result></results></report>"
            b"</create_report>",
        )

    def test_import_report_missing_report(self):
        with self.assertRaises(RequiredArgument):
            Reports.import_report(None, "task_id")

        with self.assertRaises(RequiredArgument):
            Reports.import_report("", "task_id")

    def test_import_report_missing_task_id(self):
        with self.assertRaises(RequiredArgument):
            Reports.import_report("report", None)

        with self.assertRaises(RequiredArgument):
            Reports.import_report("report", "")



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from collections.abc import Mapping, Sequence
from numbers import Integral

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import SupportsStr, deprecated, to_bool, to_comma_list
from gvm.xml import XmlCommand

from .._entity_id import EntityID
from ..v224 import HostsOrdering


class Tasks:
    @classmethod
    def clone_task(cls, task_id: EntityID) -> Request:
        """Clone an existing task

        Args:
            task_id: UUID of existing task to clone from
        """
        if not task_id:
            raise RequiredArgument(
                function=cls.clone_task.__name__, argument="task_id"
            )

        cmd = XmlCommand("create_task")
        cmd.add_element("copy", str(task_id))
        return cmd

    @classmethod
    def create_agent_group_task(
        cls,
        name: str,
        agent_group_id: EntityID,
        scanner_id: EntityID,
        *,
        comment: str | None = None,
        alterable: bool | None = None,
        schedule_id: EntityID | None = None,
        alert_ids: Sequence[EntityID] | None = None,
        schedule_periods: int | None = None,
        observers: Sequence[str] | None = None,
        preferences: Mapping[str, SupportsStr] | None = None,
    ) -> Request:
        """Create a new scan task using an agent group.

        Args:
            name: Name of the new task.
            agent_group_id: UUID of the agent group to be scanned.
            scanner_id: UUID of scanner to use for scanning the agents.
            comment: Optional comment for the task.
            alterable: Whether the task should be alterable.
            alert_ids: List of UUIDs for alerts to be applied to the task.
            schedule_id: UUID of a schedule when the task should be run.
            schedule_periods: Limit to number of scheduled runs, 0 for unlimited.
            observers: List of usernames or IDs allowed to observe the task.
            preferences: Scanner preferences as name/value pairs.
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_agent_group_task.__name__, argument="name"
            )

        if not agent_group_id:
            raise RequiredArgument(
                function=cls.create_agent_group_task.__name__,
                argument="agent_group_id",
            )

        if not scanner_id:
            raise RequiredArgument(
                function=cls.create_agent_group_task.__name__,
                argument="scanner_id",
            )

        cmd = XmlCommand("create_task")
        cmd.add_element("name", name)
        cmd.add_element("usage_type", "scan")
        cmd.add_element("agent_group", attrs={"id": str(agent_group_id)})
        cmd.add_element("scanner", attrs={"id": str(scanner_id)})

        if comment:
            cmd.add_element("comment", comment)

        if alterable is not None:
            cmd.add_element("alterable", to_bool(alterable))

        if alert_ids:
            for alert in alert_ids:
                cmd.add_element("alert", attrs={"id": str(alert)})

        if schedule_id:
            cmd.add_element("schedule", attrs={"id": str(schedule_id)})

            if schedule_periods is not None:
                if (
                    not isinstance(schedule_periods, Integral)
                    or schedule_periods < 0
                ):
                    raise InvalidArgument(
                        "schedule_periods must be an integer greater or equal than 0"
                    )
                cmd.add_element("schedule_periods", str(schedule_periods))

        if observers:
            cmd.add_element("observers", to_comma_list(observers))

        if preferences is not None:
            xml_prefs = cmd.add_element("preferences")
            for pref_name, pref_value in preferences.items():
                xml_pref = xml_prefs.add_element("preference")
                xml_pref.add_element("scanner_name", pref_name)
                xml_pref.add_element("value", str(pref_value))

        return cmd

    @classmethod
    def create_container_image_task(
        cls,
        name: str,
        oci_image_target_id: EntityID,
        scanner_id: EntityID,
        *,
        comment: str | None = None,
        alterable: bool | None = None,
        schedule_id: EntityID | None = None,
        alert_ids: Sequence[EntityID] | None = None,
        schedule_periods: int | None = None,
        observers: Sequence[str] | None = None,
        preferences: Mapping[str, SupportsStr] | None = None,
    ) -> Request:
        """Create a new scan task using an OCI image target.

        Args:
            name: Name of the new task.
            oci_image_target_id: UUID of the OCI Image target to be scanned.
            scanner_id: UUID of scanner to use for scanning the agents.
            comment: Optional comment for the task.
            alterable: Whether the task should be alterable.
            alert_ids: List of UUIDs for alerts to be applied to the task.
            schedule_id: UUID of a schedule when the task should be run.
            schedule_periods: Limit to number of scheduled runs, 0 for unlimited.
            observers: List of usernames or IDs allowed to observe the task.
            preferences: Scanner preferences as name/value pairs.
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_container_image_task.__name__,
                argument="name",
            )

        if not oci_image_target_id:
            raise RequiredArgument(
                function=cls.create_container_image_task.__name__,
                argument="oci_image_target_id",
            )

        if not scanner_id:
            raise RequiredArgument(
                function=cls.create_container_image_task.__name__,
                argument="scanner_id",
            )

        cmd = XmlCommand("create_task")
        cmd.add_element("name", name)
        cmd.add_element("usage_type", "scan")
        cmd.add_element(
            "oci_image_target", attrs={"id": str(oci_image_target_id)}
        )
        cmd.add_element("scanner", attrs={"id": str(scanner_id)})

        if comment:
            cmd.add_element("comment", comment)

        if alterable is not None:
            cmd.add_element("alterable", to_bool(alterable))

        if alert_ids:
            for alert in alert_ids:
                cmd.add_element("alert", attrs={"id": str(alert)})

        if schedule_id:
            cmd.add_element("schedule", attrs={"id": str(schedule_id)})

            if schedule_periods is not None:
                if (
                    not isinstance(schedule_periods, Integral)
                    or schedule_periods < 0
                ):
                    raise InvalidArgument(
                        "schedule_periods must be an integer greater or equal than 0"
                    )
                cmd.add_element("schedule_periods", str(schedule_periods))

        if observers:
            cmd.add_element("observers", to_comma_list(observers))

        if preferences is not None:
            xml_prefs = cmd.add_element("preferences")
            for pref_name, pref_value in preferences.items():
                xml_pref = xml_prefs.add_element("preference")
                xml_pref.add_element("scanner_name", pref_name)
                xml_pref.add_element("value", str(pref_value))

        return cmd

    @classmethod
    def create_import_task(
        cls, name: str, *, comment: str | None = None
    ) -> Request:
        """Create a new import task

        An import task is a "meta" task to import and view reports from other
        systems.

        Args:
            name: Name of the task
            comment: Comment for the task
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_import_task.__name__, argument="name"
            )

        cmd = XmlCommand("create_task")
        cmd.add_element("name", name)
        cmd.add_element("target", attrs={"id": "0"})

        if comment:
            cmd.add_element("comment", comment)

        return cmd

    @classmethod
    @deprecated(
        "The function is obsolete. Please use create_import_task instead."
    )
    def create_container_task(
        cls, name: str, *, comment: str | None = None
    ) -> "Request":
        """[DEPRECATED] Use create_import_task instead.

        This method will be removed in a future version.
        """
        return cls.create_import_task(name=name, comment=comment)

    @classmethod
    def create_task(
        cls,
        name: str,
        config_id: EntityID,
        target_id: EntityID,
        scanner_id: EntityID,
        *,
        alterable: bool | None = None,
        hosts_ordering: HostsOrdering | None = None,
        schedule_id: EntityID | None = None,
        alert_ids: Sequence[EntityID] | None = None,
        comment: str | None = None,
        schedule_periods: int | None = None,
        observers: Sequence[str] | None = None,
        preferences: Mapping[str, SupportsStr] | None = None,
    ) -> Request:
        """Create a new scan task

        Args:
            name: Name of the new task
            config_id: UUID of config to use by the task
            target_id: UUID of target to be scanned
            scanner_id: UUID of scanner to use for scanning the target
            comment: Comment for the task
            alterable: Whether the task should be alterable
            alert_ids: List of UUIDs for alerts to be applied to the task
            hosts_ordering: The order hosts are scanned in
            schedule_id: UUID of a schedule when the task should be run.
            schedule_periods: A limit to the number of times the task will be
                scheduled, or 0 for no limit
            observers: List of names or ids of users which should be allowed to
                observe this task
            preferences: Name/Value pairs of scanner preferences.
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_task.__name__, argument="name"
            )

        if not config_id:
            raise RequiredArgument(
                function=cls.create_task.__name__, argument="config_id"
            )

        if not target_id:
            raise RequiredArgument(
                function=cls.create_task.__name__, argument="target_id"
            )

        if not scanner_id:
            raise RequiredArgument(
                function=cls.create_task.__name__, argument="scanner_id"
            )

        # don't allow to create a container task with create_task
        if target_id == "0":
            raise InvalidArgument(
                function=cls.create_task.__name__, argument="target_id"
            )

        cmd = XmlCommand("create_task")
        cmd.add_element("name", name)
        cmd.add_element("usage_type", "scan")
        cmd.add_element("config", attrs={"id": str(config_id)})
        cmd.add_element("target", attrs={"id": str(target_id)})
        cmd.add_element("scanner", attrs={"id": str(scanner_id)})

        if comment:
            cmd.add_element("comment", comment)

        if alterable is not None:
            cmd.add_element("alterable", to_bool(alterable))

        if hosts_ordering:
            if not isinstance(hosts_ordering, HostsOrdering):
                hosts_ordering = HostsOrdering(hosts_ordering)
            cmd.add_element("hosts_ordering", hosts_ordering.value)

        if alert_ids:
            for alert in alert_ids:
                cmd.add_element("alert", attrs={"id": str(alert)})

        if schedule_id:
            cmd.add_element("schedule", attrs={"id": str(schedule_id)})

            if schedule_periods is not None:
                if (
                    not isinstance(schedule_periods, Integral)
                    or schedule_periods < 0
                ):
                    raise InvalidArgument(
                        "schedule_periods must be an integer greater or equal "
                        "than 0"
                    )
                cmd.add_element("schedule_periods", str(schedule_periods))

        if observers:
            # gvmd splits by comma and space
            # gvmd tries to lookup each value as user name and afterwards as
            # user id. So both user name and user id are possible
            cmd.add_element("observers", to_comma_list(observers))

        if preferences is not None:
            xml_prefs = cmd.add_element("preferences")
            for pref_name, pref_value in preferences.items():
                xml_pref = xml_prefs.add_element("preference")
                xml_pref.add_element("scanner_name", pref_name)
                xml_pref.add_element("value", str(pref_value))

        return cmd

    @classmethod
    def delete_task(
        cls, task_id: EntityID, *, ultimate: bool | None = False
    ) -> Request:
        """Deletes an existing task

        Args:
            task_id: UUID of the task to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not task_id:
            raise RequiredArgument(
                function=cls.delete_task.__name__, argument="task_id"
            )

        cmd = XmlCommand("delete_task")
        cmd.set_attribute("task_id", str(task_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))

        return cmd

    @staticmethod
    def get_tasks(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        details: bool | None = None,
        schedules_only: bool | None = None,
        ignore_pagination: bool | None = None,
    ) -> Request:
        """Request a list of tasks

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get the trashcan tasks instead
            details: Whether to include full task details
            schedules_only: Whether to only include id, name and schedule
                details
            ignore_pagination: Whether to ignore pagination settings (filter
                terms "first" and "rows"). Default is False.
        """
        cmd = XmlCommand("get_tasks")
        cmd.set_attribute("usage_type", "scan")

        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        if schedules_only is not None:
            cmd.set_attribute("schedules_only", to_bool(schedules_only))

        if ignore_pagination is not None:
            cmd.set_attribute("ignore_pagination", to_bool(ignore_pagination))

        return cmd

    @classmethod
    def get_task(cls, task_id: EntityID) -> Request:
        """Request a single task

        Args:
            task_id: UUID of an existing task
        """
        if not task_id:
            raise RequiredArgument(
                function=cls.get_task.__name__, argument="task_id"
            )

        cmd = XmlCommand("get_tasks")
        cmd.set_attribute("task_id", str(task_id))
        cmd.set_attribute("usage_type", "scan")

        # for single entity always request all details
        cmd.set_attribute("details", "1")
        return cmd

    @classmethod
    def modify_task(
        cls,
        task_id: EntityID,
        *,
        name: str | None = None,
        config_id: EntityID | None = None,
        target_id: EntityID | None = None,
        scanner_id: EntityID | None = None,
        agent_group_id: EntityID | None = None,
        oci_image_target_id: EntityID | None = None,
        alterable: bool | None = None,
        hosts_ordering: HostsOrdering | None = None,
        schedule_id: EntityID | None = None,
        schedule_periods: int | None = None,
        comment: str | None = None,
        alert_ids: Sequence[EntityID] | None = None,
        observers: Sequence[str] | None = None,
        preferences: Mapping[str, SupportsStr] | None = None,
    ) -> Request:
        """Modifies an existing task.

        Args:
            task_id: UUID of task to modify.
            name: The name of the task.
            config_id: UUID of scan config to use by the task
            target_id: UUID of target to be scanned
            scanner_id: UUID of scanner to use for scanning the target
            agent_group_id: UUID of agent group to use for scanning
            oci_image_target_id: UUID of the OCI Image target to be scanned.
            comment: The comment on the task.
            alert_ids: List of UUIDs for alerts to be applied to the task
            hosts_ordering: The order hosts are scanned in
            schedule_id: UUID of a schedule when the task should be run.
            schedule_periods: A limit to the number of times the task will be
                scheduled, or 0 for no limit.
            observers: List of names or ids of users which should be allowed to
                observe this task
            preferences: Name/Value pairs of scanner preferences.
        """
        if not task_id:
            raise RequiredArgument(
                function=cls.modify_task.__name__, argument="task_id"
            )

        if (
            sum(
                entity_id is not None
                for entity_id in (
                    target_id,
                    agent_group_id,
                    oci_image_target_id,
                )
            )
            > 1
        ):
            raise InvalidArgument(
                function=cls.modify_task.__name__,
                argument="target_id/agent_group_id/oci_image_target_id",
                message="Only one of target_id, agent_group_id or oci_image_target_id can be modified at a time",
            )

        cmd = XmlCommand("modify_task")
        cmd.set_attribute("task_id", str(task_id))

        if name:
            cmd.add_element("name", name)

        if comment:
            cmd.add_element("comment", comment)

        if config_id:
            cmd.add_element("config", attrs={"id": str(config_id)})

        if target_id:
            cmd.add_element("target", attrs={"id": str(target_id)})

        if agent_group_id:
            cmd.add_element("agent_group", attrs={"id": str(agent_group_id)})

        if oci_image_target_id:
            cmd.add_element(
                "oci_image_target", attrs={"id": str(oci_image_target_id)}
            )

        if alterable is not None:
            cmd.add_element("alterable", to_bool(alterable))

        if hosts_ordering:
            if not isinstance(hosts_ordering, HostsOrdering):
                hosts_ordering = HostsOrdering(hosts_ordering)
            cmd.add_element("hosts_ordering", hosts_ordering.value)

        if scanner_id:
            cmd.add_element("scanner", attrs={"id": str(scanner_id)})

        if schedule_id:
            cmd.add_element("schedule", attrs={"id": str(schedule_id)})

        if schedule_periods is not None:
            if (
                not isinstance(schedule_periods, Integral)
                or schedule_periods < 0
            ):
                raise InvalidArgument(
                    "schedule_periods must be an integer greater or equal "
                    "than 0"
                )
            cmd.add_element("schedule_periods", str(schedule_periods))

        if alert_ids is not None:
            if len(alert_ids) == 0:
                cmd.add_element("alert", attrs={"id": "0"})
            else:
                for alert in alert_ids:
                    cmd.add_element("alert", attrs={"id": str(alert)})

        if observers is not None:
            cmd.add_element("observers", to_comma_list(observers))

        if preferences is not None:
            xml_prefs = cmd.add_element("preferences")
            for pref_name, pref_value in preferences.items():
                xml_pref = xml_prefs.add_element("preference")
                xml_pref.add_element("scanner_name", pref_name)
                xml_pref.add_element("value", str(pref_value))

        return cmd

    @classmethod
    def move_task(
        cls, task_id: EntityID, *, slave_id: EntityID | None = None
    ) -> Request:
        """Move an existing task to another GMP slave scanner or the master

        Args:
            task_id: UUID of the task to be moved
            slave_id: UUID of the sensor to reassign the task to, empty for master.
        """
        if not task_id:
            raise RequiredArgument(
                function=cls.move_task.__name__, argument="task_id"
            )

        cmd = XmlCommand("move_task")
        cmd.set_attribute("task_id", str(task_id))

        if slave_id is not None:
            cmd.set_attribute("slave_id", str(slave_id))

        return cmd

    @classmethod
    def start_task(cls, task_id: EntityID) -> Request:
        """Start an existing task

        Args:
            task_id: UUID of the task to be started
        """
        if not task_id:
            raise RequiredArgument(
                function=cls.start_task.__name__, argument="task_id"
            )

        cmd = XmlCommand("start_task")
        cmd.set_attribute("task_id", str(task_id))
        return cmd

    @classmethod
    def resume_task(cls, task_id: EntityID) -> Request:
        """Resume an existing stopped task

        Args:
            task_id: UUID of the task to be resumed
        """
        if not task_id:
            raise RequiredArgument(
                function=cls.resume_task.__name__, argument="task_id"
            )

        cmd = XmlCommand("resume_task")
        cmd.set_attribute("task_id", str(task_id))
        return cmd

    @classmethod
    def stop_task(cls, task_id: EntityID) -> Request:
        """Stop an existing running task

        Args:
            task_id: UUID of the task to be stopped
        """
        if not task_id:
            raise RequiredArgument(
                function=cls.stop_task.__name__, argument="task_id"
            )

        cmd = XmlCommand("stop_task")
        cmd.set_attribute("task_id", str(task_id))
        return cmd



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpAuthenticateTestMixin:
    def test_missing_username(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.authenticate(None, "foo")

        with self.assertRaises(RequiredArgument):
            self.gmp.authenticate("", "foo")

    def test_missing_password(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.authenticate("bar", None)

        with self.assertRaises(RequiredArgument):
            self.gmp.authenticate("bar", "")

    def test_authentication_success(self):
        self.assertFalse(self.gmp.is_authenticated())

        self.gmp.authenticate("foo", "bar")

        self.connection.send.has_been_called_with(
            b"<authenticate>"
            b"<credentials>"
            b"<username>foo</username>"
            b"<password>bar</password>"
            b"</credentials>"
            b"</authenticate>"
        )

        self.assertTrue(self.gmp.is_authenticated())

    def test_authentication_failure(self):
        self.connection.read.return_value(
            b'<authentication_response status="400" status_text="Auth failed"/>'
        )

        self.assertFalse(self.gmp.is_authenticated())

        self.gmp.authenticate("foo", "bar")

        self.connection.send.has_been_called_with(
            b"<authenticate>"
            b"<credentials>"
            b"<username>foo</username>"
            b"<password>bar</password>"
            b"</credentials>"
            b"</authenticate>"
        )

        self.assertFalse(self.gmp.is_authenticated())



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetVersionTestCase:
    def test_get_version(self):
        self.gmp.get_version()

        self.connection.connect.has_been_called()
        self.connection.read.has_been_called()
        self.connection.send.has_been_called()
        self.connection.send.has_been_called_with(b"<get_version/>")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .permissions import (
    GmpClonePermissionTestMixin,
    GmpCreatePermissionTestMixin,
    GmpDeletePermissionTestMixin,
    GmpGetPermissionsTestMixin,
    GmpGetPermissionTestMixin,
    GmpModifyPermissionTestMixin,
)


class Gmpv224DeletePermissionTestCase(
    GmpDeletePermissionTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetPermissionTestCase(GmpGetPermissionTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetPermissionsTestCase(
    GmpGetPermissionsTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ClonePermissionTestCase(
    GmpClonePermissionTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224CreatePermissionTestCase(
    GmpCreatePermissionTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyPermissionTestCase(
    GmpModifyPermissionTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v226 import ReportFormatType


class GmpGetReportTestMixin:
    def test_get_report_without_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_report(None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_report("")

    def test_get_report_with_filter_string(self):
        self.gmp.get_report(report_id="r1", filter_string="name=foo")

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" usage_type="scan" filter="name=foo" details="1"/>'
        )

    def test_get_report_with_filter_id(self):
        self.gmp.get_report(report_id="r1", filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" usage_type="scan" filt_id="f1" details="1"/>'
        )

    def test_get_report_with_report_format_id(self):
        self.gmp.get_report(report_id="r1", report_format_id="bar")

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" usage_type="scan" format_id="bar" details="1"/>'
        )

    def test_get_report_with_report_format_id_and_config_id(self):
        self.gmp.get_report(
            report_id="r1", report_format_id="bar", report_config_id="c1"
        )

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" usage_type="scan" format_id="bar" config_id="c1" details="1"/>'
        )

    def test_get_report_with_report_format_type(self):
        self.gmp.get_report(
            report_id="r1", report_format_id=ReportFormatType.TXT
        )
        report_format_id = ReportFormatType.from_string("txt").value

        self.connection.send.has_been_called_with(
            '<get_reports report_id="r1" usage_type="scan" format_id='
            f'"{report_format_id}" details="1"/>'.encode()
        )

    def test_get_report_with_delta_report_id(self):
        self.gmp.get_report(report_id="r1", delta_report_id="r2")

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" usage_type="scan" delta_report_id="r2" details="1"/>'
        )

    def test_get_report_with_ignore_pagination(self):
        self.gmp.get_report(report_id="r1", ignore_pagination=True)

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" usage_type="scan" ignore_pagination="1" details="1"/>'
        )

        self.gmp.get_report(report_id="r1", ignore_pagination=False)

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" usage_type="scan" ignore_pagination="0" details="1"/>'
        )

    def test_get_report_with_details(self):
        self.gmp.get_report(report_id="r1", details=True)

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" usage_type="scan" details="1"/>'
        )

        self.gmp.get_report(report_id="r1", details=False)

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" usage_type="scan" details="0"/>'
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import Schedules


class SchedulesTestUnit(unittest.TestCase):
    def test_clone_schedule(self):
        request = Schedules.clone_schedule("schedule_id")
        self.assertEqual(
            bytes(request),
            b"<create_schedule><copy>schedule_id</copy></create_schedule>",
        )

    def test_clone_schedule_missing_schedule_id(self):
        with self.assertRaises(RequiredArgument):
            Schedules.clone_schedule(None)

        with self.assertRaises(RequiredArgument):
            Schedules.clone_schedule("")

    def test_create_schedule(self):
        request = Schedules.create_schedule("name", "icalendar", "timezone")
        self.assertEqual(
            bytes(request),
            b"<create_schedule>"
            b"<name>name</name>"
            b"<icalendar>icalendar</icalendar>"
            b"<timezone>timezone</timezone>"
            b"</create_schedule>",
        )

    def test_create_schedule_with_comment(self):
        request = Schedules.create_schedule(
            "name", "icalendar", "timezone", comment="comment"
        )
        self.assertEqual(
            bytes(request),
            b"<create_schedule>"
            b"<name>name</name>"
            b"<icalendar>icalendar</icalendar>"
            b"<timezone>timezone</timezone>"
            b"<comment>comment</comment>"
            b"</create_schedule>",
        )

    def test_create_schedule_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Schedules.create_schedule(None, "icalendar", "timezone")

        with self.assertRaises(RequiredArgument):
            Schedules.create_schedule("", "icalendar", "timezone")

    def test_create_schedule_missing_icalendar(self):
        with self.assertRaises(RequiredArgument):
            Schedules.create_schedule("name", None, "timezone")

        with self.assertRaises(RequiredArgument):
            Schedules.create_schedule("name", "", "timezone")

    def test_create_schedule_missing_timezone(self):
        with self.assertRaises(RequiredArgument):
            Schedules.create_schedule("name", "icalendar", None)

        with self.assertRaises(RequiredArgument):
            Schedules.create_schedule("name", "icalendar", "")

    def test_delete_schedule(self):
        request = Schedules.delete_schedule("schedule_id")
        self.assertEqual(
            bytes(request),
            b'<delete_schedule schedule_id="schedule_id" ultimate="0"/>',
        )

    def test_delete_schedule_with_ultimate(self):
        request = Schedules.delete_schedule("schedule_id", ultimate=True)
        self.assertEqual(
            bytes(request),
            b'<delete_schedule schedule_id="schedule_id" ultimate="1"/>',
        )

        request = Schedules.delete_schedule("schedule_id", ultimate=False)
        self.assertEqual(
            bytes(request),
            b'<delete_schedule schedule_id="schedule_id" ultimate="0"/>',
        )

    def test_delete_schedule_missing_schedule_id(self):
        with self.assertRaises(RequiredArgument):
            Schedules.delete_schedule(None)

        with self.assertRaises(RequiredArgument):
            Schedules.delete_schedule("")

    def test_get_schedules(self):
        request = Schedules.get_schedules()
        self.assertEqual(bytes(request), b"<get_schedules/>")

    def test_get_schedules_with_filter_string(self):
        request = Schedules.get_schedules(filter_string="filter_string")
        self.assertEqual(
            bytes(request),
            b'<get_schedules filter="filter_string"/>',
        )

    def test_get_schedules_with_filter_id(self):
        request = Schedules.get_schedules(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_schedules filt_id="filter_id"/>',
        )

    def test_get_schedules_with_trash(self):
        request = Schedules.get_schedules(trash=True)
        self.assertEqual(
            bytes(request),
            b'<get_schedules trash="1"/>',
        )

        request = Schedules.get_schedules(trash=False)
        self.assertEqual(
            bytes(request),
            b'<get_schedules trash="0"/>',
        )

    def test_get_schedules_with_tasks(self):
        request = Schedules.get_schedules(tasks=True)
        self.assertEqual(
            bytes(request),
            b'<get_schedules tasks="1"/>',
        )

        request = Schedules.get_schedules(tasks=False)
        self.assertEqual(
            bytes(request),
            b'<get_schedules tasks="0"/>',
        )

    def test_get_schedule(self):
        request = Schedules.get_schedule("schedule_id")
        self.assertEqual(
            bytes(request),
            b'<get_schedules schedule_id="schedule_id"/>',
        )

    def test_get_schedule_with_tasks(self):
        request = Schedules.get_schedule("schedule_id", tasks=True)
        self.assertEqual(
            bytes(request),
            b'<get_schedules schedule_id="schedule_id" tasks="1"/>',
        )

        request = Schedules.get_schedule("schedule_id", tasks=False)
        self.assertEqual(
            bytes(request),
            b'<get_schedules schedule_id="schedule_id" tasks="0"/>',
        )

    def test_get_schedule_missing_schedule_id(self):
        with self.assertRaises(RequiredArgument):
            Schedules.get_schedule(None)

        with self.assertRaises(RequiredArgument):
            Schedules.get_schedule("")

    def test_modify_schedule(self):
        request = Schedules.modify_schedule("schedule_id")
        self.assertEqual(
            bytes(request), b'<modify_schedule schedule_id="schedule_id"/>'
        )

    def test_modify_schedule_with_name(self):
        request = Schedules.modify_schedule("schedule_id", name="name")
        self.assertEqual(
            bytes(request),
            b'<modify_schedule schedule_id="schedule_id">'
            b"<name>name</name>"
            b"</modify_schedule>",
        )

    def test_modify_schedule_with_comment(self):
        request = Schedules.modify_schedule("schedule_id", comment="comment")
        self.assertEqual(
            bytes(request),
            b'<modify_schedule schedule_id="schedule_id">'
            b"<comment>comment</comment>"
            b"</modify_schedule>",
        )

    def test_modify_schedule_with_icalendar(self):
        request = Schedules.modify_schedule(
            "schedule_id", icalendar="icalendar"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_schedule schedule_id="schedule_id">'
            b"<icalendar>icalendar</icalendar>"
            b"</modify_schedule>",
        )

    def test_modify_schedule_with_timezone(self):
        request = Schedules.modify_schedule("schedule_id", timezone="timezone")
        self.assertEqual(
            bytes(request),
            b'<modify_schedule schedule_id="schedule_id">'
            b"<timezone>timezone</timezone>"
            b"</modify_schedule>",
        )

    def test_modify_schedule_missing_schedule_id(self):
        with self.assertRaises(RequiredArgument):
            Schedules.modify_schedule(None)

        with self.assertRaises(RequiredArgument):
            Schedules.modify_schedule("")



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v226 import UserAuthType


class GetUserAuthTypeFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            UserAuthType.from_string("foo")

    def test_none_or_empty(self):
        ct = UserAuthType.from_string(None)
        self.assertIsNone(ct)
        ct = UserAuthType.from_string("")
        self.assertIsNone(ct)

    def test_file(self):
        ct = UserAuthType.from_string("file")
        self.assertEqual(ct, UserAuthType.FILE)

    def test_radius_connect(self):
        ct = UserAuthType.from_string("radius_connect")
        self.assertEqual(ct, UserAuthType.RADIUS_CONNECT)

    def test_ldap_connect(self):
        ct = UserAuthType.from_string("ldap_connect")
        self.assertEqual(ct, UserAuthType.LDAP_CONNECT)


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import UserAuthType, Users


class UsersTestCase(unittest.TestCase):
    def test_get_users(self):
        request = Users().get_users()

        self.assertEqual(bytes(request), b"<get_users/>")

    def test_get_users_with_filter_string(self):
        request = Users().get_users(filter_string="foo=bar")

        self.assertEqual(bytes(request), b'<get_users filter="foo=bar"/>')

    def test_get_users_with_filter_id(self):
        request = Users().get_users(filter_id="f1")

        self.assertEqual(bytes(request), b'<get_users filt_id="f1"/>')

    def test_create_user(self):
        request = Users().create_user(name="foo")

        self.assertEqual(
            bytes(request), b"<create_user><name>foo</name></create_user>"
        )

    def test_create_user_with_password(self):
        request = Users().create_user(name="foo", password="bar")

        self.assertEqual(
            bytes(request),
            b"<create_user><name>foo</name><password>bar</password></create_user>",
        )

    def test_create_user_with_hosts(self):
        request = Users().create_user(
            name="foo", hosts=["h1", "h2"], hosts_allow=True
        )

        self.assertEqual(
            bytes(request),
            b"<create_user>"
            b"<name>foo</name>"
            b'<hosts allow="1">h1,h2</hosts>'
            b"</create_user>",
        )

        request = Users().create_user(
            name="foo", hosts=["h1", "h2"], hosts_allow=False
        )

        self.assertEqual(
            bytes(request),
            b"<create_user>"
            b"<name>foo</name>"
            b'<hosts allow="0">h1,h2</hosts>'
            b"</create_user>",
        )

    def test_create_user_with_role_ids(self):
        request = Users().create_user(name="foo", role_ids=["r1", "r2"])

        self.assertEqual(
            bytes(request),
            b"<create_user>"
            b"<name>foo</name>"
            b'<role id="r1"/>'
            b'<role id="r2"/>'
            b"</create_user>",
        )

    def test_create_user_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Users().create_user("")

        with self.assertRaises(RequiredArgument):
            Users().create_user(None)

    def test_modify_user(self):
        request = Users().modify_user(user_id="u1")

        self.assertEqual(bytes(request), b'<modify_user user_id="u1"/>')

    def test_modify_user_missing_user_id(self):
        with self.assertRaises(RequiredArgument):
            Users().modify_user(user_id=None)

        with self.assertRaises(RequiredArgument):
            Users().modify_user(user_id="")

    def test_modify_user_with_new_name(self):
        request = Users().modify_user(user_id="u1", name="foo")

        self.assertEqual(
            bytes(request),
            b'<modify_user user_id="u1"><new_name>foo</new_name></modify_user>',
        )

    def test_modify_user_with_new_comment(self):
        request = Users().modify_user(user_id="u1", comment="foo")

        self.assertEqual(
            bytes(request),
            b'<modify_user user_id="u1"><comment>foo</comment></modify_user>',
        )

    def test_modify_user_with_role_ids(self):
        request = Users().modify_user(user_id="u1", role_ids=[])

        self.assertEqual(bytes(request), b'<modify_user user_id="u1"/>')

        request = Users().modify_user(user_id="u1", role_ids=["r1"])

        self.assertEqual(
            bytes(request),
            b'<modify_user user_id="u1"><role id="r1"/></modify_user>',
        )

        request = Users().modify_user(user_id="u1", role_ids=["r1", "r2"])

        self.assertEqual(
            bytes(request),
            b'<modify_user user_id="u1"><role id="r1"/><role id="r2"/></modify_user>',
        )

    def test_modify_user_with_group_ids(self):
        request = Users().modify_user(user_id="u1", group_ids=[])

        self.assertEqual(bytes(request), b'<modify_user user_id="u1"/>')

        request = Users().modify_user(user_id="u1", group_ids=["g1"])

        self.assertEqual(
            bytes(request),
            b'<modify_user user_id="u1"><groups><group id="g1"/></groups></modify_user>',
        )

        request = Users().modify_user(user_id="u1", group_ids=["g1", "g2"])

        self.assertEqual(
            bytes(request),
            b'<modify_user user_id="u1"><groups><group id="g1"/><group id="g2"/></groups></modify_user>',
        )

    def test_modify_user_with_password(self):
        request = Users().modify_user(user_id="u1", password="bar")

        self.assertEqual(
            bytes(request),
            b'<modify_user user_id="u1"><password>bar</password></modify_user>',
        )

    def test_modify_user_with_auth_source(self):
        request = Users().modify_user(
            user_id="u1", auth_source=UserAuthType.LDAP_CONNECT
        )

        self.assertEqual(
            bytes(request),
            b'<modify_user user_id="u1">'
            b"<sources><source>ldap_connect</source></sources>"
            b"</modify_user>",
        )

        request = Users().modify_user(user_id="u1", auth_source="ldap_connect")

        self.assertEqual(
            bytes(request),
            b'<modify_user user_id="u1">'
            b"<sources><source>ldap_connect</source></sources>"
            b"</modify_user>",
        )

    def test_modify_user_invalid_auth_source(self):
        with self.assertRaises(InvalidArgument):
            Users().modify_user(user_id="u1", auth_source="foo")

    def test_modify_user_with_hosts(self):
        request = Users().modify_user(
            user_id="u1", hosts=["h1", "h2"], hosts_allow=True
        )

        self.assertEqual(
            bytes(request),
            b'<modify_user user_id="u1">'
            b'<hosts allow="1">h1,h2</hosts>'
            b"</modify_user>",
        )

        request = Users().modify_user(
            user_id="u1", hosts=["h1", "h2"], hosts_allow=False
        )

        self.assertEqual(
            bytes(request),
            b'<modify_user user_id="u1">'
            b'<hosts allow="0">h1,h2</hosts>'
            b"</modify_user>",
        )

    def test_clone_user(self):
        request = Users().clone_user(user_id="u1")

        self.assertEqual(
            bytes(request), b"<create_user><copy>u1</copy></create_user>"
        )

    def test_clone_user_missing_user_id(self):
        with self.assertRaises(RequiredArgument):
            Users().clone_user(user_id=None)

        with self.assertRaises(RequiredArgument):
            Users().clone_user(user_id="")

    def test_delete_user_with_user_id(self):
        request = Users().delete_user("u1")

        self.assertEqual(bytes(request), b'<delete_user user_id="u1"/>')

    def test_delete_user_with_name(self):
        request = Users().delete_user(name="foo")

        self.assertEqual(bytes(request), b'<delete_user name="foo"/>')

    def test_delete_user_with_inheritor_id(self):
        request = Users().delete_user("u1", inheritor_id="u2")

        self.assertEqual(
            bytes(request), b'<delete_user user_id="u1" inheritor_id="u2"/>'
        )

    def test_delete_user_with_inheritor_name(self):
        request = Users().delete_user("u1", inheritor_name="foo")

        self.assertEqual(
            bytes(request), b'<delete_user user_id="u1" inheritor_name="foo"/>'
        )

    def test_delete_user_missing_user_id_and_name(self):
        with self.assertRaises(RequiredArgument):
            Users().delete_user(None)

        with self.assertRaises(RequiredArgument):
            Users().delete_user("")

        with self.assertRaises(RequiredArgument):
            Users().delete_user(user_id="", name="")

    def test_get_user(self):
        request = Users().get_user("u1")

        self.assertEqual(bytes(request), b'<get_users user_id="u1"/>')

    def test_get_user_missing_user_id(self):
        with self.assertRaises(RequiredArgument):
            Users().get_user(user_id=None)

        with self.assertRaises(RequiredArgument):
            Users().get_user("")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import Tags


class TagsTestCase(unittest.TestCase):
    def test_clone_tag(self):
        request = Tags.clone_tag("tag_id")
        self.assertEqual(
            bytes(request),
            b"<create_tag><copy>tag_id</copy></create_tag>",
        )

    def test_clone_tag_missing_tag_id(self):
        with self.assertRaises(RequiredArgument):
            Tags.clone_tag(None)

        with self.assertRaises(RequiredArgument):
            Tags.clone_tag("")

    def test_create_tag(self):
        request = Tags.create_tag("name", "alert")
        self.assertEqual(
            bytes(request),
            b"<create_tag>"
            b"<name>name</name>"
            b"<resources>"
            b"<type>alert</type>"
            b"</resources>"
            b"</create_tag>",
        )

    def test_create_tag_with_resource_filter(self):
        request = Tags.create_tag("name", "alert", resource_filter="filter")
        self.assertEqual(
            bytes(request),
            b"<create_tag>"
            b"<name>name</name>"
            b'<resources filter="filter">'
            b"<type>alert</type>"
            b"</resources>"
            b"</create_tag>",
        )

    def test_create_tag_with_resource_ids(self):
        request = Tags.create_tag("name", "alert", resource_ids=["id1", "id2"])
        self.assertEqual(
            bytes(request),
            b"<create_tag>"
            b"<name>name</name>"
            b"<resources>"
            b'<resource id="id1"/>'
            b'<resource id="id2"/>'
            b"<type>alert</type>"
            b"</resources>"
            b"</create_tag>",
        )

    def test_create_tag_with_value(self):
        request = Tags.create_tag("name", "alert", value="value")
        self.assertEqual(
            bytes(request),
            b"<create_tag>"
            b"<name>name</name>"
            b"<resources>"
            b"<type>alert</type>"
            b"</resources>"
            b"<value>value</value>"
            b"</create_tag>",
        )

    def test_create_tag_with_comment(self):
        request = Tags.create_tag("name", "alert", comment="comment")
        self.assertEqual(
            bytes(request),
            b"<create_tag>"
            b"<name>name</name>"
            b"<resources>"
            b"<type>alert</type>"
            b"</resources>"
            b"<comment>comment</comment>"
            b"</create_tag>",
        )

    def test_create_tag_with_active(self):
        request = Tags.create_tag("name", "alert", active=True)
        self.assertEqual(
            bytes(request),
            b"<create_tag>"
            b"<name>name</name>"
            b"<resources>"
            b"<type>alert</type>"
            b"</resources>"
            b"<active>1</active>"
            b"</create_tag>",
        )

        request = Tags.create_tag("name", "alert", active=False)
        self.assertEqual(
            bytes(request),
            b"<create_tag>"
            b"<name>name</name>"
            b"<resources>"
            b"<type>alert</type>"
            b"</resources>"
            b"<active>0</active>"
            b"</create_tag>",
        )

    def test_create_tag_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Tags.create_tag(None, "alert")

        with self.assertRaises(RequiredArgument):
            Tags.create_tag("", "alert")

    def test_create_tag_missing_resource_type(self):
        with self.assertRaises(RequiredArgument):
            Tags.create_tag("name", None)

        with self.assertRaises(RequiredArgument):
            Tags.create_tag("name", "")

    def test_create_tag_invalid_resource_type(self):
        with self.assertRaises(InvalidArgument):
            Tags.create_tag("name", "invalid")

    def test_create_tag_invalid_resource_filter_and_resource_ids(self):
        with self.assertRaises(InvalidArgument):
            Tags.create_tag(
                "name",
                "alert",
                resource_filter="filter",
                resource_ids=["id1", "id2"],
            )

    def test_delete_tag(self):
        request = Tags.delete_tag("tag_id")
        self.assertEqual(
            bytes(request),
            b'<delete_tag tag_id="tag_id" ultimate="0"/>',
        )

    def test_delete_tag_with_ultimate(self):
        request = Tags.delete_tag("tag_id", ultimate=True)
        self.assertEqual(
            bytes(request),
            b'<delete_tag tag_id="tag_id" ultimate="1"/>',
        )

        request = Tags.delete_tag("tag_id", ultimate=False)
        self.assertEqual(
            bytes(request),
            b'<delete_tag tag_id="tag_id" ultimate="0"/>',
        )

    def test_get_tags(self):
        request = Tags.get_tags()
        self.assertEqual(
            bytes(request),
            b"<get_tags/>",
        )

    def test_get_tags_with_filter_string(self):
        request = Tags.get_tags(filter_string="filter")
        self.assertEqual(
            bytes(request),
            b'<get_tags filter="filter"/>',
        )

    def test_get_tags_with_filter_id(self):
        request = Tags.get_tags(filter_id="filter")
        self.assertEqual(
            bytes(request),
            b'<get_tags filt_id="filter"/>',
        )

    def test_get_tags_with_trash(self):
        request = Tags.get_tags(trash=True)
        self.assertEqual(
            bytes(request),
            b'<get_tags trash="1"/>',
        )

        request = Tags.get_tags(trash=False)
        self.assertEqual(
            bytes(request),
            b'<get_tags trash="0"/>',
        )

    def test_get_tags_with_names_only(self):
        request = Tags.get_tags(names_only=True)
        self.assertEqual(
            bytes(request),
            b'<get_tags names_only="1"/>',
        )

        request = Tags.get_tags(names_only=False)
        self.assertEqual(
            bytes(request),
            b'<get_tags names_only="0"/>',
        )

    def test_get_tag(self):
        request = Tags.get_tag("tag_id")
        self.assertEqual(
            bytes(request),
            b'<get_tags tag_id="tag_id"/>',
        )

    def test_get_tag_missing_tag_id(self):
        with self.assertRaises(RequiredArgument):
            Tags.get_tag(None)

        with self.assertRaises(RequiredArgument):
            Tags.get_tag("")

    def test_modify_tag(self):
        request = Tags.modify_tag("tag_id", resource_type="alert")
        self.assertEqual(
            bytes(request),
            b'<modify_tag tag_id="tag_id">'
            b"<resources>"
            b"<type>alert</type>"
            b"</resources>"
            b"</modify_tag>",
        )

    def test_modify_tag_with_comment(self):
        request = Tags.modify_tag(
            "tag_id", resource_type="alert", comment="comment"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_tag tag_id="tag_id">'
            b"<comment>comment</comment>"
            b"<resources>"
            b"<type>alert</type>"
            b"</resources>"
            b"</modify_tag>",
        )

    def test_modify_tag_with_name(self):
        request = Tags.modify_tag("tag_id", resource_type="alert", name="name")
        self.assertEqual(
            bytes(request),
            b'<modify_tag tag_id="tag_id">'
            b"<name>name</name>"
            b"<resources>"
            b"<type>alert</type>"
            b"</resources>"
            b"</modify_tag>",
        )

    def test_modify_tag_with_value(self):
        request = Tags.modify_tag(
            "tag_id", resource_type="alert", value="value"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_tag tag_id="tag_id">'
            b"<value>value</value>"
            b"<resources>"
            b"<type>alert</type>"
            b"</resources>"
            b"</modify_tag>",
        )

    def test_modify_tag_with_active(self):
        request = Tags.modify_tag("tag_id", resource_type="alert", active=True)
        self.assertEqual(
            bytes(request),
            b'<modify_tag tag_id="tag_id">'
            b"<active>1</active>"
            b"<resources>"
            b"<type>alert</type>"
            b"</resources>"
            b"</modify_tag>",
        )

        request = Tags.modify_tag("tag_id", resource_type="alert", active=False)
        self.assertEqual(
            bytes(request),
            b'<modify_tag tag_id="tag_id">'
            b"<active>0</active>"
            b"<resources>"
            b"<type>alert</type>"
            b"</resources>"
            b"</modify_tag>",
        )

    def test_modify_tag_with_resource_action(self):
        request = Tags.modify_tag("tag_id", resource_action="add")
        self.assertEqual(
            bytes(request),
            b'<modify_tag tag_id="tag_id">'
            b'<resources action="add"/>'
            b"</modify_tag>",
        )

    def test_modify_tag_with_resource_filter(self):
        request = Tags.modify_tag(
            "tag_id", resource_type="alert", resource_filter="filter"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_tag tag_id="tag_id">'
            b'<resources filter="filter">'
            b"<type>alert</type>"
            b"</resources>"
            b"</modify_tag>",
        )

    def test_modify_tag_with_resource_ids(self):
        request = Tags.modify_tag("tag_id", resource_ids=["id1", "id2"])
        self.assertEqual(
            bytes(request),
            b'<modify_tag tag_id="tag_id">'
            b"<resources>"
            b'<resource id="id1"/>'
            b'<resource id="id2"/>'
            b"</resources>"
            b"</modify_tag>",
        )

    def test_modify_tag_missing_tag_id(self):
        with self.assertRaises(RequiredArgument):
            Tags.modify_tag(None)

        with self.assertRaises(RequiredArgument):
            Tags.modify_tag("")

    def test_modify_tag_with_resource_filter_missing_resource_type(self):
        with self.assertRaises(RequiredArgument):
            Tags.modify_tag("tag_id", resource_filter="filter")

        with self.assertRaises(RequiredArgument):
            Tags.modify_tag(
                "tag_id", resource_filter="filter", resource_type=None
            )

        with self.assertRaises(RequiredArgument):
            Tags.modify_tag(
                "tag_id", resource_filter="filter", resource_type=""
            )

    def test_modify_tag_invalid_resource_type(self):
        with self.assertRaises(InvalidArgument):
            Tags.modify_tag("tag_id", resource_type="invalid")



# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from decimal import Decimal

from gvm.errors import InvalidArgument, RequiredArgument


class GmpModifyNoteTestMixin:
    def test_modify_note(self):
        self.gmp.modify_note(note_id="n1", text="foo")

        self.connection.send.has_been_called_with(
            b'<modify_note note_id="n1"><text>foo</text></modify_note>'
        )

    def test_modify_note_missing_note_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_note(note_id=None, text="foo")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_note(note_id="", text="foo")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_note("", text="foo")

    def test_modify_note_missing_text(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_note(note_id="n1", text="")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_note(note_id="n1", text=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_note("n1", "")

    def test_modify_note_with_days_active(self):
        self.gmp.modify_note(note_id="n1", text="foo", days_active=0)

        self.connection.send.has_been_called_with(
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<active>0</active>"
            b"</modify_note>"
        )

        self.gmp.modify_note(note_id="n1", text="foo", days_active=-1)

        self.connection.send.has_been_called_with(
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<active>-1</active>"
            b"</modify_note>"
        )

        self.gmp.modify_note(note_id="n1", text="foo", days_active=600)

        self.connection.send.has_been_called_with(
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<active>600</active>"
            b"</modify_note>"
        )

    def test_modify_note_with_port(self):
        self.gmp.modify_note(note_id="n1", text="foo", port="123/tcp")

        self.connection.send.has_been_called_with(
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<port>123/tcp</port>"
            b"</modify_note>"
        )

    def test_modify_note_with_hosts(self):
        self.gmp.modify_note(note_id="n1", text="foo", hosts=["foo"])

        self.connection.send.has_been_called_with(
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<hosts>foo</hosts>"
            b"</modify_note>"
        )

        self.gmp.modify_note(note_id="n1", text="foo", hosts=["foo", "bar"])

        self.connection.send.has_been_called_with(
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<hosts>foo,bar</hosts>"
            b"</modify_note>"
        )

    def test_modify_note_with_result_id(self):
        self.gmp.modify_note(note_id="n1", text="foo", result_id="r1")

        self.connection.send.has_been_called_with(
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b'<result id="r1"/>'
            b"</modify_note>"
        )

    def test_modify_note_with_task_id(self):
        self.gmp.modify_note(note_id="n1", text="foo", task_id="r1")

        self.connection.send.has_been_called_with(
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b'<task id="r1"/>'
            b"</modify_note>"
        )

    def test_modify_note_with_severity(self):
        self.gmp.modify_note(note_id="n1", text="foo", severity="5.5")

        self.connection.send.has_been_called_with(
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<severity>5.5</severity>"
            b"</modify_note>"
        )

        self.gmp.modify_note(note_id="n1", text="foo", severity=5.5)

        self.connection.send.has_been_called_with(
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<severity>5.5</severity>"
            b"</modify_note>"
        )

        self.gmp.modify_note(note_id="n1", text="foo", severity=Decimal("5.5"))

        self.connection.send.has_been_called_with(
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<severity>5.5</severity>"
            b"</modify_note>"
        )

    def test_modify_note_with_invalid_port(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_note(note_id="o1", text="foo", port="123")

        with self.assertRaises(InvalidArgument):
            self.gmp.modify_note(note_id="o1", text="foo", port="tcp/123")



# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from gvm.protocols.gmp import GMPv224

from .. import GmpTestCase


class Gmpv224TestCase(GmpTestCase):
    gmp_class = GMPv224



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from typing import Any

from gvm.protocols._protocol import GvmProtocol
from tests import CallableMock


class MockConnection:
    def __init__(self):
        self.connect = CallableMock("connect")
        self.disconnect = CallableMock("disconnect")
        self.send = CallableMock("send")
        self.read = CallableMock("read")
        self.read.return_value(b'<foo_response status="200"/>')
        self.finish_send = CallableMock("finish_send")


class GmpTestCase(unittest.TestCase):
    gmp_class: type[GvmProtocol[Any]]

    def setUp(self):
        self.connection = MockConnection()
        # pylint: disable=not-callable
        self.gmp = self.gmp_class(self.connection)



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.tasks import (
    GmpCloneTaskTestMixin,
    GmpCreateContainerTaskTestMixin,
    GmpCreateTaskTestMixin,
    GmpDeleteTaskTestMixin,
    GmpGetTasksTestMixin,
    GmpGetTaskTestMixin,
    GmpModifyTaskTestMixin,
    GmpMoveTaskTestMixin,
    GmpResumeTaskTestMixin,
    GmpStartTaskTestMixin,
    GmpStopTaskTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225CloneTaskTestCase(GmpCloneTaskTestMixin, GMPTestCase):
    pass


class Gmpv225CreateContainerTaskTestCase(
    GmpCreateContainerTaskTestMixin, GMPTestCase
):
    pass


class Gmpv225CreateTaskTestCase(GmpCreateTaskTestMixin, GMPTestCase):
    pass


class Gmpv225DeleteTaskTestCase(GmpDeleteTaskTestMixin, GMPTestCase):
    pass


class Gmpv225GetTaskTestCase(GmpGetTaskTestMixin, GMPTestCase):
    pass


class Gmpv225GetTasksTestCase(GmpGetTasksTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyTaskTestCase(GmpModifyTaskTestMixin, GMPTestCase):
    pass


class Gmpv225MoveTaskTestCase(GmpMoveTaskTestMixin, GMPTestCase):
    pass


class Gmpv225ResumeTaskTestCase(GmpResumeTaskTestMixin, GMPTestCase):
    pass


class Gmpv225StartTaskTestCase(GmpStartTaskTestMixin, GMPTestCase):
    pass


class Gmpv225StopTaskTestCase(GmpStopTaskTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.tasks import (
    GmpCloneTaskTestMixin,
    GmpCreateContainerTaskTestMixin,
    GmpCreateTaskTestMixin,
    GmpDeleteTaskTestMixin,
    GmpGetTasksTestMixin,
    GmpGetTaskTestMixin,
    GmpModifyTaskTestMixin,
    GmpMoveTaskTestMixin,
    GmpResumeTaskTestMixin,
    GmpStartTaskTestMixin,
    GmpStopTaskTestMixin,
)
from ...gmpv226 import GMPTestCase


class GMPCloneTaskTestCase(GmpCloneTaskTestMixin, GMPTestCase):
    pass


class GMPCreateContainerTaskTestCase(
    GmpCreateContainerTaskTestMixin, GMPTestCase
):
    pass


class GMPCreateTaskTestCase(GmpCreateTaskTestMixin, GMPTestCase):
    pass


class GMPDeleteTaskTestCase(GmpDeleteTaskTestMixin, GMPTestCase):
    pass


class GMPGetTaskTestCase(GmpGetTaskTestMixin, GMPTestCase):
    pass


class GMPGetTasksTestCase(GmpGetTasksTestMixin, GMPTestCase):
    pass


class GMPModifyTaskTestCase(GmpModifyTaskTestMixin, GMPTestCase):
    pass


class GMPMoveTaskTestCase(GmpMoveTaskTestMixin, GMPTestCase):
    pass


class GMPResumeTaskTestCase(GmpResumeTaskTestMixin, GMPTestCase):
    pass


class GMPStartTaskTestCase(GmpStartTaskTestMixin, GMPTestCase):
    pass


class GMPStopTaskTestCase(GmpStopTaskTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.system.authentication import (
    GmpAuthenticateTestMixin,
    GmpDescribeAuthTestMixin,
    GmpModifyAuthTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225AuthenticateTestCase(GmpAuthenticateTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyAuthTestCase(GmpModifyAuthTestMixin, GMPTestCase):
    pass


class Gmpv225DescribeAuthCommandTestCase(GmpDescribeAuthTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.targets import (
    GmpCloneTargetTestMixin,
    GmpCreateTargetTestMixin,
    GmpDeleteTargetTestMixin,
    GmpGetTargetsTestMixin,
    GmpGetTargetTestMixin,
    GmpModifyTargetTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225CloneTargetTestCase(GmpCloneTargetTestMixin, GMPTestCase):
    pass


class Gmpv225CreateTargetTestCase(GmpCreateTargetTestMixin, GMPTestCase):
    pass


class Gmpv225DeleteTargetTestCase(GmpDeleteTargetTestMixin, GMPTestCase):
    pass


class Gmpv225GetTargetTestCase(GmpGetTargetTestMixin, GMPTestCase):
    pass


class Gmpv225GetTargetsTestCase(GmpGetTargetsTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyTargetTestCase(GmpModifyTargetTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import AliveTest


class GmpModifyTargetTestMixin:
    def test_modify_target(self):
        self.gmp.modify_target(target_id="t1")

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1"/>'
        )

    def test_modify_target_missing_target_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_target(target_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_target(target_id="")

    def test_modify_target_with_comment(self):
        self.gmp.modify_target(target_id="t1", comment="foo")

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<comment>foo</comment>"
            b"</modify_target>"
        )

    def test_modify_target_with_hosts(self):
        self.gmp.modify_target(target_id="t1", hosts=["foo"])

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<hosts>foo</hosts>"
            b"<exclude_hosts></exclude_hosts>"
            b"</modify_target>"
        )

        self.gmp.modify_target(target_id="t1", hosts=["foo", "bar"])

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<hosts>foo,bar</hosts>"
            b"<exclude_hosts></exclude_hosts>"
            b"</modify_target>"
        )

    def test_modify_target_with_hosts_and_exclude_hosts(self):
        self.gmp.modify_target(
            target_id="t1", hosts=["foo", "bar"], exclude_hosts=["foo"]
        )

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<hosts>foo,bar</hosts>"
            b"<exclude_hosts>foo</exclude_hosts>"
            b"</modify_target>"
        )

    def test_modify_target_with_name(self):
        self.gmp.modify_target(target_id="t1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1"><name>foo</name></modify_target>'
        )

    def test_modify_target_with_exclude_hosts(self):
        self.gmp.modify_target(target_id="t1", exclude_hosts=["foo"])

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<exclude_hosts>foo</exclude_hosts>"
            b"</modify_target>"
        )

        self.gmp.modify_target(target_id="t1", exclude_hosts=["foo", "bar"])

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<exclude_hosts>foo,bar</exclude_hosts>"
            b"</modify_target>"
        )

    def test_modify_target_with_ssh_credential(self):
        self.gmp.modify_target(target_id="t1", ssh_credential_id="c1")

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b'<ssh_credential id="c1"/>'
            b"</modify_target>"
        )

    def test_modify_target_with_ssh_credential_port(self):
        self.gmp.modify_target(
            target_id="t1", ssh_credential_id="c1", ssh_credential_port=123
        )

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b'<ssh_credential id="c1">'
            b"<port>123</port>"
            b"</ssh_credential>"
            b"</modify_target>"
        )

    def test_modify_target_with_smb_credential_id(self):
        self.gmp.modify_target(target_id="t1", smb_credential_id="c1")

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b'<smb_credential id="c1"/>'
            b"</modify_target>"
        )

    def test_modify_target_with_esxi_credential_id(self):
        self.gmp.modify_target(target_id="t1", esxi_credential_id="c1")

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b'<esxi_credential id="c1"/>'
            b"</modify_target>"
        )

    def test_modify_target_with_snmp_credential_id(self):
        self.gmp.modify_target(target_id="t1", snmp_credential_id="c1")

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b'<snmp_credential id="c1"/>'
            b"</modify_target>"
        )

    def test_modify_target_with_alive_tests(self):
        self.gmp.modify_target(target_id="t1", alive_test=AliveTest.ICMP_PING)

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<alive_tests>ICMP Ping</alive_tests>"
            b"</modify_target>"
        )

    def test_modify_target_invalid_alive_tests(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_target(target_id="t1", alive_test="foo")

    def test_modify_target_with_allow_simultaneous_ips(self):
        self.gmp.modify_target(target_id="t1", allow_simultaneous_ips=True)

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<allow_simultaneous_ips>1</allow_simultaneous_ips>"
            b"</modify_target>"
        )

        self.gmp.modify_target(target_id="t1", allow_simultaneous_ips=False)

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<allow_simultaneous_ips>0</allow_simultaneous_ips>"
            b"</modify_target>"
        )

    def test_modify_target_with_reverse_lookup_only(self):
        self.gmp.modify_target(target_id="t1", reverse_lookup_only=True)

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<reverse_lookup_only>1</reverse_lookup_only>"
            b"</modify_target>"
        )

        self.gmp.modify_target(target_id="t1", reverse_lookup_only=False)

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<reverse_lookup_only>0</reverse_lookup_only>"
            b"</modify_target>"
        )

    def test_modify_target_with_reverse_lookup_unify(self):
        self.gmp.modify_target(target_id="t1", reverse_lookup_unify=True)

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<reverse_lookup_unify>1</reverse_lookup_unify>"
            b"</modify_target>"
        )

        self.gmp.modify_target(target_id="t1", reverse_lookup_unify=False)

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b"<reverse_lookup_unify>0</reverse_lookup_unify>"
            b"</modify_target>"
        )

    def test_modify_target_with_port_list_id(self):
        self.gmp.modify_target(target_id="t1", port_list_id="pl1")

        self.connection.send.has_been_called_with(
            b'<modify_target target_id="t1">'
            b'<port_list id="pl1"/>'
            b"</modify_target>"
        )



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import GvmError


class GmpGetTLSCertificateTestMixin:
    def test_get_tls_certificate(self):
        self.gmp.get_tls_certificate("t1")

        self.connection.send.has_been_called_with(
            b'<get_tls_certificates tls_certificate_id="t1" '
            b'include_certificate_data="1" details="1"/>'
        )

    def test_fail_without_tls_certificate_id(self):
        with self.assertRaises(GvmError):
            self.gmp.get_tls_certificate(None)

        with self.assertRaises(GvmError):
            self.gmp.get_tls_certificate("")



# SPDX-FileCopyrightText: 2023-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v227 import AlertCondition


class GetAlertConditionFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            AlertCondition.from_string("foo")

    def test_none_or_empty(self):
        ct = AlertCondition.from_string(None)
        self.assertIsNone(ct)
        ct = AlertCondition.from_string("")
        self.assertIsNone(ct)

    def test_always(self):
        ct = AlertCondition.from_string("always")
        self.assertEqual(ct, AlertCondition.ALWAYS)

    def test_filter_count_at_least(self):
        ct = AlertCondition.from_string("filter count at least")
        self.assertEqual(ct, AlertCondition.FILTER_COUNT_AT_LEAST)

    def test_filter_count_changed(self):
        ct = AlertCondition.from_string("filter count changed")
        self.assertEqual(ct, AlertCondition.FILTER_COUNT_CHANGED)

    def test_severity_at_least(self):
        ct = AlertCondition.from_string("severity at least")
        self.assertEqual(ct, AlertCondition.SEVERITY_AT_LEAST)

    def test_severity_changed(self):
        ct = AlertCondition.from_string("severity changed")
        self.assertEqual(ct, AlertCondition.SEVERITY_CHANGED)

    def test_error(self):
        ct = AlertCondition.from_string("error")
        self.assertEqual(ct, AlertCondition.ERROR)



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""
http client for initializing a connection to the openvasd HTTP API using optional mTLS authentication.
"""

import ssl
from os import PathLike

from httpx import Client

StrOrPathLike = str | PathLike[str]


def create_openvasd_http_client(
    host_name: str,
    *,
    api_key: str | None = None,
    server_ca_path: StrOrPathLike | None = None,
    client_cert_paths: StrOrPathLike
    | tuple[StrOrPathLike, StrOrPathLike]
    | None = None,
    port: int = 3000,
) -> Client:
    """
    Create a `httpx.Client` configured for mTLS-secured or API KEY access
    to an openvasd HTTP API instance.

    Args:
        host_name: Hostname or IP of the OpenVASD server (e.g., "localhost").
        api_key: Optional API key used for authentication via HTTP headers.
        server_ca_path: Path to the server's CA certificate (for verifying the server).
        client_cert_paths: Path to the client certificate (str) or a tuple of
                            (cert_path, key_path) for mTLS authentication.
        port: The port to connect to (default: 3000).

    Behavior:
        - If both `server_ca_path` and `client_cert_paths` are set, an mTLS connection
            is established using an SSLContext.
        - If not, `verify` is set to False (insecure), and HTTP is used instead of HTTPS.
            HTTP connection needs api_key for authorization.
    """
    headers = {}

    context: ssl.SSLContext | None = None

    # Prepare mTLS SSL context if needed
    if client_cert_paths and server_ca_path:
        context = ssl.create_default_context(
            ssl.Purpose.SERVER_AUTH, cafile=server_ca_path
        )
        if isinstance(client_cert_paths, tuple):
            context.load_cert_chain(
                certfile=client_cert_paths[0], keyfile=client_cert_paths[1]
            )
        else:
            context.load_cert_chain(certfile=client_cert_paths)

        context.check_hostname = False
        context.verify_mode = ssl.CERT_REQUIRED

    # Set verify based on context presence
    verify: bool | ssl.SSLContext = context if context else False

    if api_key:
        headers["X-API-KEY"] = api_key

    protocol = "https" if context else "http"
    base_url = f"{protocol}://{host_name}:{port}"

    return Client(
        base_url=base_url,
        headers=headers,
        verify=verify,
        http2=True,
        timeout=10.0,
    )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpModifyOperatingSystemTestMixin:
    def test_modify_operating_system(self):
        self.gmp.modify_operating_system(operating_system_id="a1")

        self.connection.send.has_been_called_with(
            b'<modify_asset asset_id="a1"><comment></comment></modify_asset>'
        )

    def test_modify_operating_system_without_operating_system_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_operating_system(
                operating_system_id=None, comment="foo"
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_operating_system(
                operating_system_id="", comment="foo"
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_operating_system("", comment="foo")

    def test_modify_operating_system_with_comment(self):
        self.gmp.modify_operating_system("a1", comment="foo")

        self.connection.send.has_been_called_with(
            b'<modify_asset asset_id="a1"><comment>foo</comment></modify_asset>'
        )

        self.gmp.modify_operating_system("a1", comment="foo")

        self.connection.send.has_been_called_with(
            b'<modify_asset asset_id="a1"><comment>foo</comment></modify_asset>'
        )

        self.gmp.modify_operating_system("a1", comment="")

        self.connection.send.has_been_called_with(
            b'<modify_asset asset_id="a1"><comment></comment></modify_asset>'
        )

        self.gmp.modify_operating_system("a1", comment=None)

        self.connection.send.has_been_called_with(
            b'<modify_asset asset_id="a1"><comment></comment></modify_asset>'
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .notes import (
    GmpCloneNoteTestMixin,
    GmpCreateNoteTestMixin,
    GmpDeleteNoteTestMixin,
    GmpGetNotesTestMixin,
    GmpGetNoteTestMixin,
    GmpModifyNoteTestMixin,
)


class Gmpv224DeleteNoteTestCase(GmpDeleteNoteTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetNoteTestCase(GmpGetNoteTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetNotesTestCase(GmpGetNotesTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CloneNoteTestCase(GmpCloneNoteTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreateNoteTestCase(GmpCreateNoteTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyNoteTestCase(GmpModifyNoteTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from lxml import etree

from gvm.errors import GvmError
from gvm.transforms import EtreeCheckCommandTransform


class EtreeCheckCommandTransformTestCase(unittest.TestCase):
    def test_no_status_transform(self):
        transform = EtreeCheckCommandTransform()

        with self.assertRaises(GvmError):
            transform(b"<foo/>")

    def test_no_success_status_transform(self):
        transform = EtreeCheckCommandTransform()

        root = etree.Element("foo_response")
        root.set("status", "400")
        root.set("status_text", "Foo error")

        response = etree.tostring(root)

        with self.assertRaises(GvmError):
            transform(response)

    def test_success_response(self):
        transform = EtreeCheckCommandTransform()

        root = etree.Element("foo_response")
        root.set("status", "200")

        response = etree.tostring(root)

        result = transform(response)

        self.assertTrue(etree.iselement(result))
        self.assertEqual(result.tag, "foo_response")
        self.assertEqual(result.get("status"), "200")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_tag import GmpCloneTagTestMixin
from .test_create_tag import GmpCreateTagTestMixin
from .test_delete_tag import GmpDeleteTagTestMixin
from .test_get_tag import GmpGetTagTestMixin
from .test_get_tags import GmpGetTagsTestMixin
from .test_modify_tag import GmpModifyTagTestMixin

__all__ = (
    "GmpCloneTagTestMixin",
    "GmpCreateTagTestMixin",
    "GmpDeleteTagTestMixin",
    "GmpGetTagTestMixin",
    "GmpGetTagsTestMixin",
    "GmpModifyTagTestMixin",
)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import base64
import errno
import hashlib
import logging
import socket as socketlib
import sys
from collections.abc import Callable
from os import PathLike
from pathlib import Path
from time import time
from typing import Any, TextIO

import paramiko
import paramiko.ssh_exception
import paramiko.transport

from gvm.errors import GvmError

from ._connection import BUF_SIZE, DEFAULT_TIMEOUT

logger = logging.getLogger("gvm.connections.ssh")

DEFAULT_SSH_PORT = 22
DEFAULT_SSH_USERNAME = "gmp"
DEFAULT_SSH_PASSWORD = ""
DEFAULT_HOSTNAME = "127.0.0.1"
DEFAULT_KNOWN_HOSTS_FILE = ".ssh/known_hosts"


class SSHConnection:
    """
    SSH Class to connect, read and write from GVM via SSH

    """

    def __init__(
        self,
        *,
        timeout: int | float | None = DEFAULT_TIMEOUT,
        hostname: str | None = DEFAULT_HOSTNAME,
        port: int | None = DEFAULT_SSH_PORT,
        username: str | None = DEFAULT_SSH_USERNAME,
        password: str | None = DEFAULT_SSH_PASSWORD,
        known_hosts_file: str | PathLike | None = None,
        auto_accept_host: bool | None = None,
        file: TextIO = sys.stdout,
        input: Callable[[], str] = input,  # noqa: A002
        exit: Callable[[str], Any] = sys.exit,  # noqa: A002
    ) -> None:
        """
        Create a new SSH connection instance.

        Args:
            timeout: Timeout in seconds for the connection.
            hostname: DNS name or IP address of the remote server. Default is
                127.0.0.1.
            port: Port of the remote SSH server. Default is port 22.
            username: Username to use for SSH login. Default is "gmp".
            password: Password to use for SSH login. Default is "".
        """
        self._client: paramiko.SSHClient | None = None
        self.hostname = hostname if hostname is not None else DEFAULT_HOSTNAME
        self.port = int(port) if port is not None else DEFAULT_SSH_PORT
        self.username = (
            username if username is not None else DEFAULT_SSH_USERNAME
        )
        self.password = (
            password if password is not None else DEFAULT_SSH_PASSWORD
        )
        self.known_hosts_file = (
            Path(known_hosts_file)
            if known_hosts_file is not None
            else Path.home() / DEFAULT_KNOWN_HOSTS_FILE
        )
        self.auto_accept_host = auto_accept_host
        self._timeout = timeout
        self._file = file
        self._input = input
        self._exit = exit

    def _send_all(self, data: bytes) -> int:
        """Returns the sum of sent bytes if success"""
        sent_sum = 0
        while data:
            sent = self._stdin.channel.send(data)

            if not sent:
                # Connection was closed by server
                raise GvmError("Remote closed the connection")

            sent_sum += sent

            data = data[sent:]
        return sent_sum

    def _auto_accept_host(
        self, hostkeys: paramiko.HostKeys, key: paramiko.PKey
    ) -> None:
        if self.port == DEFAULT_SSH_PORT:
            hostkeys.add(self.hostname, key.get_name(), key)
        elif self.port != DEFAULT_SSH_PORT:
            hostkeys.add(
                "[" + self.hostname + "]:" + str(self.port),
                key.get_name(),
                key,
            )
        try:
            hostkeys.save(filename=str(self.known_hosts_file))
        except OSError as e:
            raise GvmError(
                "Something went wrong with writing "
                f"the known_hosts file {self.known_hosts_file.absolute()}: {e}"
            ) from None

        key_type = key.get_name().replace("ssh-", "").upper()

        logger.info(
            "Warning: Permanently added '%s' (%s) to the list of known hosts.",
            self.hostname,
            key_type,
        )

    def _ssh_authentication_input_loop(
        self, hostkeys: paramiko.HostKeys, key: paramiko.PKey
    ) -> None:
        # Ask user for permission to continue
        # let it look like openssh
        sha64_fingerprint = base64.b64encode(
            hashlib.sha256(base64.b64decode(key.get_base64())).digest()
        ).decode("utf-8")[:-1]
        key_type = key.get_name().replace("ssh-", "").upper()

        print(
            f"The authenticity of host '{self.hostname}' can't be established.",
            file=self._file,
        )
        print(
            f"{key_type} key fingerprint is {sha64_fingerprint}.",
            file=self._file,
        )
        print(
            "Are you sure you want to continue connecting (yes/no)? ",
            end="",
            file=self._file,
        )

        add = self._input()
        while True:
            if add == "yes":
                if self.port == DEFAULT_SSH_PORT:
                    hostkeys.add(self.hostname, key.get_name(), key)
                elif self.port != DEFAULT_SSH_PORT:
                    hostkeys.add(
                        "[" + self.hostname + "]:" + str(self.port),
                        key.get_name(),
                        key,
                    )

                # ask user if the key should be added permanently
                print(
                    f"Do you want to add {self.hostname} "
                    "to known_hosts (yes/no)? ",
                    end="",
                    file=self._file,
                )

                save = self._input()
                while True:
                    if save == "yes":
                        try:
                            hostkeys.save(filename=str(self.known_hosts_file))
                        except OSError as e:
                            raise GvmError(
                                "Something went wrong with writing "
                                f"the known_hosts file: {e}"
                            ) from None

                        logger.info(
                            "Warning: Permanently added '%s' (%s) to "
                            "the list of known hosts.",
                            self.hostname,
                            key_type,
                        )
                        break
                    elif save == "no":
                        logger.info(
                            "Warning: Host '%s' (%s) not added to "
                            "the list of known hosts.",
                            self.hostname,
                            key_type,
                        )
                        break
                    else:
                        print(
                            "Please type 'yes' or 'no': ",
                            end="",
                            file=self._file,
                        )
                        save = self._input()
                break
            elif add == "no":
                self._exit("User denied key. Host key verification failed.")
            else:
                print("Please type 'yes' or 'no': ", end="", file=self._file)
                add = self._input()

    def _get_remote_host_key(self) -> paramiko.PKey:
        """Get the remote host key for ssh connection"""
        try:
            tmp_socket = socketlib.socket()
            tmp_socket.settimeout(self._timeout)
            tmp_socket.connect((self.hostname, self.port))
        except OSError as e:
            tmp_socket.close()
            raise GvmError(
                "Couldn't establish a connection to fetch the"
                f" remote server key: {e}"
            ) from None

        trans = paramiko.transport.Transport(tmp_socket)
        try:
            trans.start_client()
        except paramiko.SSHException as e:
            tmp_socket.close()
            raise GvmError(
                f"Couldn't fetch the remote server key: {e}"
            ) from None

        key = trans.get_remote_server_key()
        try:
            trans.close()
        except paramiko.SSHException as e:
            raise GvmError(
                f"Couldn't close the connection to the remote server key: {e}"
            ) from None
        finally:
            tmp_socket.close()

        return key

    def _ssh_authentication(self) -> None:
        """Search/add/save the servers key for the SSH authentication process"""

        if not self._client:
            raise GvmError("SSH Client not connected.")

        # set to reject policy (avoid MITM attacks)
        self._client.set_missing_host_key_policy(paramiko.RejectPolicy())

        # openssh is posix, so this might only a posix approach
        # https://stackoverflow.com/q/32945533
        try:
            # load the keys into paramiko and check if remote is in the list
            self._client.load_host_keys(filename=str(self.known_hosts_file))
        except OSError as e:
            if e.errno != errno.ENOENT:
                raise GvmError(
                    "Something went wrong with reading "
                    f"the known_hosts file: {e}"
                ) from None

        hostkeys = self._client.get_host_keys()

        # Switch based on SSH Port
        if self.port == DEFAULT_SSH_PORT:
            hostname = self.hostname
        else:
            hostname = f"[{self.hostname}]:{self.port}"

        if not hostkeys.lookup(hostname):
            # Key not found, so connect to remote and fetch the key
            # with the paramiko Transport protocol
            key = self._get_remote_host_key()
            if self.auto_accept_host:
                self._auto_accept_host(hostkeys=hostkeys, key=key)
            else:
                self._ssh_authentication_input_loop(hostkeys=hostkeys, key=key)

    def _read(self) -> bytes:
        return self._stdout.channel.recv(BUF_SIZE)

    def send(self, data: bytes) -> None:
        self._send_all(data)

    def read(self) -> bytes:
        break_timeout = (
            time() + self._timeout if self._timeout is not None else None
        )

        data = self._read()

        if not data:
            # Connection was closed by server
            raise GvmError("Remote closed the connection")

        if break_timeout and time() > break_timeout:
            raise GvmError("Timeout while reading the response")

        return data

    def connect(self) -> None:
        """
        Connect to the SSH server and authenticate to it
        """
        self._client = paramiko.SSHClient()
        self._ssh_authentication()

        try:
            self._client.connect(
                hostname=self.hostname,
                username=self.username,
                password=self.password,
                timeout=self._timeout,
                port=int(self.port),
                allow_agent=False,
                look_for_keys=False,
            )

        except (
            paramiko.BadHostKeyException,
            paramiko.AuthenticationException,
            paramiko.SSHException,
            paramiko.ssh_exception.NoValidConnectionsError,
            ConnectionError,
        ) as e:
            raise GvmError(f"SSH Connection failed: {e}") from None

        self._stdin, self._stdout, self._stderr = self._client.exec_command(
            "", get_pty=False
        )

    def disconnect(self) -> None:
        """Disconnect and close the connection to the remote server"""
        try:
            if self._client is not None:
                self._client.close()
        except OSError as e:
            logger.debug("Connection closing error: %s", e)
            raise e

        if self._client is not None:
            self._client = None
            del self._stdin, self._stdout, self._stderr

    def finish_send(self) -> None:
        # shutdown socket for sending. only allow reading data afterwards
        self._stdout.channel.shutdown(socketlib.SHUT_WR)



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .user_settings import (
    GmpGetUserSettingsTestMixin,
    GmpGetUserSettingTestMixin,
    GmpModifyUserSettingTestMixin,
)


class Gmpv224GetUserSettingTestCase(
    GmpGetUserSettingTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetUserSettingsTestCase(
    GmpGetUserSettingsTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyUserSettingTestCase(
    GmpModifyUserSettingTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.roles import (
    GmpCloneRoleTestMixin,
    GmpCreateRoleTestMixin,
    GmpDeleteRoleTestMixin,
    GmpGetRolesTestMixin,
    GmpGetRoleTestMixin,
    GmpModifyRoleTestMixin,
)
from ...gmpv226 import GMPTestCase


class GMPDeleteRoleTestCase(GmpDeleteRoleTestMixin, GMPTestCase):
    pass


class GMPGetRoleTestCase(GmpGetRoleTestMixin, GMPTestCase):
    pass


class GMPGetRolesTestCase(GmpGetRolesTestMixin, GMPTestCase):
    pass


class GMPCloneRoleTestCase(GmpCloneRoleTestMixin, GMPTestCase):
    pass


class GMPCreateRoleTestCase(GmpCreateRoleTestMixin, GMPTestCase):
    pass


class GMPModifyRoleTestCase(GmpModifyRoleTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool, to_comma_list
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class Roles:
    @classmethod
    def clone_role(cls, role_id: EntityID) -> Request:
        """Clone an existing role

        Args:
            role_id: UUID of an existing role to clone from
        """
        if not role_id:
            raise RequiredArgument(
                function=cls.clone_role.__name__, argument="role_id"
            )

        cmd = XmlCommand("create_role")
        cmd.add_element("copy", str(role_id))
        return cmd

    @classmethod
    def create_role(
        cls,
        name: str,
        *,
        comment: str | None = None,
        users: list[str] | None = None,
    ) -> Request:
        """Create a new role

        Args:
            name: Name of the role
            comment: Comment for the role
            users: List of user names to add to the role
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_role.__name__, argument="name"
            )

        cmd = XmlCommand("create_role")
        cmd.add_element("name", name)

        if comment:
            cmd.add_element("comment", comment)

        if users:
            cmd.add_element("users", to_comma_list(users))

        return cmd

    @classmethod
    def delete_role(
        cls, role_id: str, *, ultimate: bool | None = False
    ) -> Request:
        """Deletes an existing role

        Args:
            role_id: UUID of the role to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not role_id:
            raise RequiredArgument(
                function=cls.delete_role.__name__, argument="role_id"
            )

        cmd = XmlCommand("delete_role")
        cmd.set_attribute("role_id", role_id)
        cmd.set_attribute("ultimate", to_bool(ultimate))
        return cmd

    @staticmethod
    def get_roles(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
    ) -> Request:
        """Request a list of roles

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get the trashcan roles instead
        """
        cmd = XmlCommand("get_roles")

        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        return cmd

    @classmethod
    def get_role(cls, role_id: EntityID) -> Request:
        """Request a single role

        Args:
            role_id: UUID of an existing role
        """
        if not role_id:
            raise RequiredArgument(
                function=cls.get_role.__name__, argument="role_id"
            )

        cmd = XmlCommand("get_roles")
        cmd.set_attribute("role_id", str(role_id))
        return cmd

    @classmethod
    def modify_role(
        cls,
        role_id: EntityID,
        *,
        comment: str | None = None,
        name: str | None = None,
        users: list[str] | None = None,
    ) -> Request:
        """Modifies an existing role.

        Args:
            role_id: UUID of role to modify.
            comment: Name of role.
            name: Comment on role.
            users: List of user names.
        """
        if not role_id:
            raise RequiredArgument(
                function=cls.modify_role.__name__, argument="role_id argument"
            )

        cmd = XmlCommand("modify_role")
        cmd.set_attribute("role_id", str(role_id))

        if comment:
            cmd.add_element("comment", comment)

        if name:
            cmd.add_element("name", name)

        if users:
            cmd.add_element("users", to_comma_list(users))

        return cmd
